/* Copyright (C) 1997, 1998 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/*
 *  ISO C 9X:  7.8 Complex arithmetic <complex.h>
 */

#ifndef _COMPLEX_
#define _COMPLEX_  1

#include <features.h>

/* Get general and ISO C 9X specific information.  */
#include <bits/mathdef.h>

__BEGIN_DECLS

/* We might need to add support for more compilers here.  But once ISO
   C 9X is out hopefully all maintained compilers will provide the data
   types `float complex' and `double complex'.  */
#if (__GNUC__ == 2 && __GNUC_MINOR__ >= 7) || __GNUC__ > 2
# define _Complex __complex__
#endif


/* Narrowest imaginary unit.  This depends on the floating-point
   evaluation method.
   XXX This probably has to go into a gcc related file.  */
#define _Complex_I  (1.0iF)

/* Another more descriptive name is `I'.
   XXX Once we have the imaginary support switch this to _Imaginary_I.  */
#undef I
#define I _Complex_I


/* Optimization aids.  This is not yet implemented in gcc and once it
   is this will probably be available in a gcc header.  */
#define CX_LIMITED_RANGE_ON
#define CX_LIMITED_RANGE_OFF
#define CX_LIMITED_RANGE_DEFAULT


#if defined __GNUC__ && (__GNUC__ > 2 || __GNUC__ == 2 && __GNUC_MINOR__ >= 7)

# ifdef __NO_LONG_DOUBLE_MATH

#  define __mathfunc1(function) \
extern __complex__ double function __P ((__complex__ double __x)); \
extern __complex__ float function##f __P ((__complex__ float __x)); \
__EXTERN_INLINE __complex__ long double function##l (__complex__ long double __x) \
  { return function ((__complex__ double)__x); } \
extern __complex__ double __##function __P ((__complex__ double __x)); \
extern __complex__ float __##function##f __P ((__complex__ float __x)); \
__EXTERN_INLINE __complex__ long double __##function##l (__complex__ long double __x) \
  { return function ((__complex__ double)__x); }

#  define __mathfunc2(function) \
extern __complex__ double function __P ((__complex__ double __x, __complex__ double __y)); \
extern __complex__ float function##f __P ((__complex__ float __x, __complex__ float __y)); \
__EXTERN_INLINE __complex__ long double function##l (__complex__ long double __x, __complex__ long double __y) \
  { return function ((__complex__ double)__x, (__complex__ double)__y); } \
extern __complex__ double __##function __P ((__complex__ double __x, __complex__ double __y)); \
extern __complex__ float __##function##f __P ((__complex__ float __x, __complex__ float __y)); \
__EXTERN_INLINE __complex__ long double __##function##l (__complex__ long double __x, __complex__ long double __y) \
  { return function ((__complex__ double)__x, (__complex__ double)__y); }

#  define __mathfunc_real(function) \
extern double function __P ((__complex__ double __x)); \
extern float function##f __P ((__complex__ float __x)); \
__EXTERN_INLINE long double function##l (__complex__ long double __x) { return function ((__complex__ double)__x); } \
extern double __##function __P ((__complex__ double __x)); \
extern float __##function##f __P ((__complex__ float __x)); \
__EXTERN_INLINE long double __##function##l (__complex__ long double __x) { return function ((__complex__ double)__x); }

# else

#  define __mathfunc1(function) \
extern __complex__ float function##f __P ((__complex__ float __x)); \
extern __complex__ double function __P ((__complex__ double __x)); \
extern __complex__ long double function##l __P ((__complex__ long double __x)); \
extern __complex__ float __##function##f __P ((__complex__ float __x)); \
extern __complex__ double __##function __P ((__complex__ double __x)); \
extern __complex__ long double __##function##l __P ((__complex__ long double __x));

#  define __mathfunc2(function) \
extern __complex__ float function##f __P ((__complex__ float __x, __complex__ float __y)); \
extern __complex__ double function __P ((__complex__ double __x, __complex__ double __y)); \
extern __complex__ long double function##l __P ((__complex__ long double __x, __complex__ long double __y)); \
extern __complex__ float __##function##f __P ((__complex__ float __x, __complex__ float __y)); \
extern __complex__ double __##function __P ((__complex__ double __x, __complex__ double __y)); \
extern __complex__ long double __##function##l __P ((__complex__ long double __x, __complex__ long double __y));

#  define __mathfunc_real(function) \
extern float function##f __P ((__complex__ float __x)); \
extern double function __P ((__complex__ double __x)); \
extern long double function##l __P ((__complex__ long double __x)); \
extern float __##function##f __P ((__complex__ float __x)); \
extern double __##function __P ((__complex__ double __x)); \
extern long double __##function##l __P ((__complex__ long double __x));

# endif

#else

typedef struct __complex__ {
  double __real__, __imag__;
} __complex_t__;

typedef struct __complexf__ {
  float __real__, __imag__;
} __complexf_t__;

typedef struct __complexl__ {
  long double __real__, __imag__;
} __complexl_t__;

# ifdef __NO_LONG_DOUBLE_MATH

#  define __mathfunc1(function) \
extern __complex_t__ function __P ((__complex_t__ __x)); \
extern __complexf_t__ function##f __P ((__complexf_t__ __x)); \
__EXTERN_INLINE __complexl_t__ function##l (__complexl_t____x) { return function ((__complex_t__)__x); } \
extern __complex_t__ __##function __P ((__complex_t__ __x)); \
extern __complexf_t__ __##function##f __P ((__complexf_t__ __x)); \
__EXTERN_INLINE __complexl_t__ __##function##l (__complexl_t____x) { return function ((__complex_t__)__x); }

#  define __mathfunc2(function) \
extern __complex_t__ function __P ((__complex_t__ __x, __complex_t__ __y)); \
extern __complexf_t__ function##f __P ((__complexf_t__ __x, __complexf_t__ __y)); \
__EXTERN_INLINE __complexl_t__ function##l (__complexl_t__ __x, __complexl_t__ __y) \
  { return function ((__complex_t__)__x, (__complex_t__)__y); } \
extern __complex_t__ __##function __P ((__complex_t__ __x, __complex_t__ __y)); \
extern __complexf_t__ __##function##f __P ((__complexf_t__ __x, __complexf_t__ __y)); \
__EXTERN_INLINE __complexl_t__ function##l (__complexl_t__ __x, __complexl_t__ __y) \
  { return function ((__complex_t__)__x, (__complex_t__)__y); }

#  define __mathfunc_real(function) \
extern double function __P ((__complex_t__ __x)); \
extern float function##f __P ((__complexf_t__ __x)); \
__EXTERN_INLINE long double function##l (__complexl_t____x) { return function ((__complex_t__)__x); }

# else

#  define __mathfunc1(function) \
extern __complexf_t__ function##f __P ((__complexf_t__ __x)); \
extern __complex_t__ function __P ((__complex_t__ __x)); \
extern __complexl_t__ function##l __P ((__complexl_t__ __x));

#  define __mathfunc2(function) \
extern __complexf_t__ function##f __P ((__complexf_t__ __x, __complexf_t__ __y)); \
extern __complex_t__ function __P ((__complex_t__ __x, __complex_t__ __y)); \
extern __complexl_t__  function##l __P ((__complexl_t__ __x, __complexl_t__ __y));

#  define __mathfunc_real(function) \
extern float function##f __P ((__complexf_t__ __x)); \
extern double function __P ((__complex_t__ __x)); \
extern long double function##l __P ((__complexl_t__ __x));

# endif

#endif

/* Trigonometric functions.  */

/* Arc cosine of Z.  */
__mathfunc1 (cacos);
/* Arc sine of Z.  */
__mathfunc1 (casin);
/* Arc tangent of Z.  */
__mathfunc1 (catan);

/* Cosine of Z.  */
__mathfunc1 (ccos);
/* Sine of Z.  */
__mathfunc1 (csin);
/* Tangent of Z.  */
__mathfunc1 (ctan);


/* Hyperbolic functions.  */

/* Hyperbolic arc cosine of Z.  */
__mathfunc1 (cacosh);
/* Hyperbolic arc sine of Z.  */
__mathfunc1 (casinh);
/* Hyperbolic arc tangent of Z.  */
__mathfunc1 (catanh);

/* Hyperbolic cosine of Z.  */
__mathfunc1 (ccosh);
/* Hyperbolic sine of Z.  */
__mathfunc1 (csinh);
/* Hyperbolic tangent of Z.  */
__mathfunc1 (ctanh);


/* Exponential and logarithmic functions.  */

/* Exponential function of Z.  */
__mathfunc1 (cexp);

/* Natural logarithm of Z.  */
__mathfunc1 (clog);

#ifdef __USE_GNU
/* The base 10 logarithm is not defined by the standard but to implement
   the standard C++ library it is handy.  */
__mathfunc1 (clog10);
#endif

/* Power functions.  */

/* Return X to the Y power.  */
__mathfunc2 (cpow);

/* Return the square root of Z.  */
__mathfunc1 (csqrt);


/* Absolute value, conjugates, and projection.  */

/* Absolute value of Z.  */
__mathfunc_real (cabs);

/* Argument value of Z.  */
__mathfunc_real (carg);

/* Complex conjugate of Z.  */
__mathfunc1 (conj);

/* Projection of Z onto the Riemann sphere.  */
__mathfunc1 (cproj);


/* Decomposing complex values.  */

/* Imaginary part of Z.  */
__mathfunc_real (cimag);

/* Real part of Z.  */
__mathfunc_real (creal);

#undef __mathfunc_real
#undef __mathfunc2
#undef __mathfunc1

/* Now some optimized versions.  GCC has handy notations for these
   functions.  */
#if defined __GNUC__ && defined __OPTIMIZE__ && !defined __NO_MATH_INLINES

#  define __mathfunc_real(function, code) \
__EXTERN_INLINE double function (__complex__ double __x) { code; } \
__EXTERN_INLINE float function##f (__complex__ float __x) { code; } \
__EXTERN_INLINE long double function##l (__complex__ long double __x) { code; } \
__EXTERN_INLINE double __##function (__complex__ double __x) { code; } \
__EXTERN_INLINE float __##function##f (__complex__ float __x) { code; } \
__EXTERN_INLINE long double __##function##l (__complex__ long double __x) { code; }

#  define __mathfunc(function, code) \
__EXTERN_INLINE __complex__ double function (__complex__ double __x) { code; } \
__EXTERN_INLINE __complex__ float function##f (__complex__ float __x) { code; } \
__EXTERN_INLINE __complex__ long double function##l (__complex__ long double __x) { code; } \
__EXTERN_INLINE __complex__ double __##function (__complex__ double __x) { code; } \
__EXTERN_INLINE __complex__ float __##function##f (__complex__ float __x) { code; } \
__EXTERN_INLINE __complex__ long double __##function##l (__complex__ long double __x) { code; }


/* Imaginary part of Z. */
__mathfunc_real (cimag, return __imag__ __x);

/* Real part of Z.  */
__mathfunc_real (creal, return __real__ __x);

/* Complex conjugate of Z.  */
__mathfunc (conj, return ~__z);

#undef __mathfunc_real
#undef __mathfunc

# if defined __GNUC__ && (__GNUC__ > 2 || __GNUC__ == 2 && __GNUC_MINOR__ >= 7)

__EXTERN_INLINE double __cabs (__complex__ double z) {
  return hypot (__real__ z, __imag__ z);
}

__EXTERN_INLINE double cabs (__complex__ double z) {
  return hypot (__real__ z, __imag__ z);
}

__EXTERN_INLINE float cabsf (__complex__ float z) {
  return hypotf (__real__ z, __imag__ z);
}

__EXTERN_INLINE float cabsf (__complex__ float z) {
  return hypotf (__real__ z, __imag__ z);
}

__EXTERN_INLINE long double cabsl (__complex__ long double z) {
  return hypotl (__real__ z, __imag__ z);
}

__EXTERN_INLINE long double cabsl (__complex__ long double z) {
  return hypotl (__real__ z, __imag__ z);
}

__EXTERN_INLINE __complex__ double __cacos (__complex__ double x) {
  __complex__ double y;
  __complex__ double res;

  y = casin (x);

  __real__ res = (double) M_PI_2 - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__EXTERN_INLINE __complex__ double cacos (__complex__ double x) {
  __complex__ double y;
  __complex__ double res;

  y = casin (x);

  __real__ res = (double) M_PI_2 - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__EXTERN_INLINE __complex__ float __cacosf (__complex__ float x) {
  __complex__ float y;
  __complex__ float res;

  y = casinf (x);

  __real__ res = (float) M_PI_2 - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__EXTERN_INLINE __complex__ float cacosf (__complex__ float x) {
  __complex__ float y;
  __complex__ float res;

  y = casinf (x);

  __real__ res = (float) M_PI_2 - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__EXTERN_INLINE __complex__ long double __cacosl (__complex__ long double x) {
  __complex__ long double y;
  __complex__ long double res;

  y = casinl (x);

  __real__ res = M_PI_2l - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__EXTERN_INLINE __complex__ long double cacosl (__complex__ long double x) {
  __complex__ long double y;
  __complex__ long double res;

  y = casinl (x);

  __real__ res = M_PI_2l - __real__ y;
  __imag__ res = -__imag__ y;

  return res;
}

__complex__ double __cpow (__complex__ double x, __complex__ double c) {
  return cexp (c * clog (x));
}

__complex__ double cpow (__complex__ double x, __complex__ double c) {
  return cexp (c * clog (x));
}

__complex__ float __cpowf (__complex__ float x, __complex__ float c) {
  return cexpf (c * clogf (x));
}

__complex__ float cpowf (__complex__ float x, __complex__ float c) {
  return cexpf (c * clogf (x));
}

__complex__ long double __cpowl (__complex__ long double x, __complex__ long double c) {
  return cexpl (c * clogl (x));
}

__complex__ long double cpowl (__complex__ long double x, __complex__ long double c) {
  return cexpl (c * clogl (x));
}

# endif
#endif

__END_DECLS

#endif /* complex.h */
