/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * ISO C 9X 7.6: Floating-point environment <fenv.h>
 */

#ifndef _FENV_
#define _FENV_

#include <features.h>

__BEGIN_DECLS

/* Get the architecture dependend definitions.  The following definitions
   are expected to be done:

   fenv_t type for object representing an entire floating-point
    environment

   FE_DFL_ENV macro of type pointer to fenv_t to be used as the argument
    to functions taking an argument of type fenv_t; in this
    case the default environment will be used

   fexcept_t  type for object representing the floating-point exception
    flags including status associated with the flags

   The following macros are defined iff the implementation supports this
   kind of exception.
   FE_INEXACT   inexact result
   FE_DIVBYZERO   devision by zero
   FE_UNDERFLOW   result not representable due to underflow
   FE_OVERFLOW    result not representable due to overflow
   FE_INVALID   invalid operation

   FE_ALL_EXCEPT  bitwise OR of all supported exceptions

   The next macros are defined iff the appropriate rounding mode is
   supported by the implementation.
   FE_TONEAREST   round to nearest
   FE_UPWARD    round toward +Inf
   FE_DOWNWARD    round toward -Inf
   FE_TOWARDZERO  round toward 0
*/
#include <bits/fenv.h>

/* Floating-point exception handling.  */

/* Clear the supported exceptions represented by EXCEPTS.  */
extern void feclearexcept __P ((int excepts));

/* Store implementation-defined representation of the exception flags
   indicated by EXCEPTS in the object pointed to by FLAGP.  */
extern void fegetexceptflag __P ((fexcept_t *flagp, int excepts));

/* Raise the supported exceptions represented by EXCEPTS.  */
extern void feraiseexcept __P ((int excepts));

/* Set complete status for exceptions indicated by EXCEPTS according to
   the representation in the object pointed to by FLAGP.  */
extern void fesetexceptflag __P ((const fexcept_t *flagp, int excepts));

/* Determine which of subset of the exceptions specified by EXCEPTS are
   currently set.  */
extern int fetestexcept __P ((int excepts));


/* Rounding control.  */

/* Get current rounding direction.  */
extern int fegetround __P ((void));

/* Establish the rounding direction represented by ROUND.  */
extern int fesetround __P ((int rounding_direction));


/* Rounding precision control.  */

extern int fegetprec (
  void
);

extern int fesetprec (
  int precision
);

/* Floating-point environment.  */

/* Store the current floating-point environment in the object pointed
   to by ENVP.  */
extern void fegetenv __P ((fenv_t *envp));

/* Save the current environment in the object pointed to by ENVP, clear
   exception flags and install a non-stop mode (if available) for all
   exceptions.  */
extern int feholdexcept __P ((fenv_t *envp));

/* Establish the floating-point environment represented by the object
   pointed to by ENVP.  */
extern void fesetenv __P ((const fenv_t *envp));

/* Save current exceptions in temporary storage, install environment
   represented by object pointed to by ENVP and raise exceptions
   according to saved exceptions.  */
extern void feupdateenv __P ((const fenv_t *envp));

#ifdef __USE_GNU

/* Enable individual exceptions.  Will not enable more exceptions than
   EXCEPTS specifies.  Returns the previous enabled exceptions if all
   exceptions are successfully set, otherwise returns -1.  */
extern int feenableexcept __P((int excepts));

/* Disable individual exceptions.  Will not disable more exceptions than
   EXCEPTS specifies.  Returns the previous enabled exceptions if all
   exceptions are successfully disabled, otherwise returns -1.  */
extern int fedisableexcept __P((int __excepts));

/* Return enabled exceptions.  */
extern int fegetexcept __P((void));
#endif

__END_DECLS

#endif /* fenv.h */
