/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _HEAP_
#define _HEAP_

#include <features.h>
#include <sys/types.h>

#ifndef BASETYPES
# define BASETYPES
typedef unsigned long ULONG;
typedef ULONG *PULONG;
typedef unsigned short USHORT;
typedef USHORT *PUSHORT;
typedef unsigned char UCHAR;
typedef UCHAR *PUCHAR;
typedef char *PSZ;
#endif  /* !BASETYPES */

typedef void *PVOID;
#define VOID void
typedef int BOOL;
typedef unsigned char BYTE;

#ifdef _WIN64
typedef unsigned __int64 SIZE_T;
#else
typedef unsigned long SIZE_T;
#endif

#define HEAPAPI __stdcall
#ifndef __GNUC__
# define HEAPREGPARM __fastcall
#else
# define HEAPREGPARM __attribute__ ((regparm (3)))
#endif
#ifndef EXTERNAPI
# define EXTERNAPI __declspec (dllimport)
#endif

__BEGIN_DECLS

#pragma pack (push, 4)

enum {
  RtHeapCallHandler          =   2,
  RtHeapGenerateSignal       =   4,
  RtHeapZeroMemory           =   8,
  /* for RtHeapReAlloc only */
  RtHeapInplaceOnly          =  16,
  /* for RtHeapCreate, RtHeapReAlloc and RtHeapWalk */
  RtHeapNoDecommit           =  32,
  /* the next two for RtHeapWalk only */
  RtHeapFreeBlock            =  64,
  RtHeapStopWalk             = 128,
  /* for RtHeapReAlloc only */
  RtHeapDiscardBlock         = 256,
};

typedef struct _HEAP *HEAP;

#ifndef _STRUCT_HEAPDEFINED
struct _HEAP {
  ULONG Dummy;
};
#endif

/* All function set _doserrno if not say else */
EXTERNAPI HEAP HEAPAPI RtHeapCreate __P((
  ULONG Flags,
  SIZE_T InitSize
));

EXTERNAPI void HEAPAPI RtHeapDestroy __P((
  HEAP Heap
));

EXTERNAPI PVOID HEAPAPI RtHeapAlloc __P((
  HEAP Heap,
  ULONG Flags,
  SIZE_T Size,
  ULONG Align
));

EXTERNAPI PVOID HEAPAPI RtHeapAllocDbg __P((
  HEAP Heap,
  ULONG Flags,
  SIZE_T Size,
  ULONG Align,
  const char *FileName,
  int LineNumber
));

EXTERNAPI VOID HEAPAPI RtHeapFree __P((
  HEAP Heap,
  PVOID MemAddr
));

EXTERNAPI VOID HEAPAPI RtHeapFreeDbg __P((
  HEAP Heap,
  PVOID MemAddr
));

EXTERNAPI PVOID HEAPAPI RtHeapReAlloc __P((
  HEAP Heap,
  ULONG Flags,
  PVOID MemAddr,
  SIZE_T Size,
  ULONG Align
));

EXTERNAPI PVOID HEAPAPI RtHeapReAllocDbg __P((
  HEAP Heap,
  ULONG Flags,
  PVOID MemAddr,
  SIZE_T Size,
  ULONG Align,
  const char *FileName,
  int LineNumber
));

/* if trimSize then return really trimmed Size */
/* else return Size of the biggest free block */
EXTERNAPI SIZE_T HEAPAPI RtHeapCompact __P((
  HEAP Heap,
  ULONG Flags,
  SIZE_T TrimSize
));

/* return Size in Bytes if block was allocated from the Heap or 0 */
EXTERNAPI SIZE_T HEAPAPI RtHeapUsableSize __P((
  HEAP Heap,
  PVOID MemAddr
));

EXTERNAPI SIZE_T HEAPAPI RtHeapUsableSizeDbg __P((
  HEAP Heap,
  PVOID MemAddr
));

typedef struct _BLOCK_DESC {
  PVOID BlkAddr;
  SIZE_T BlkSize;
  BOOL BlkFree;
#ifndef NDEBUG
  PVOID BlkRealAddr;
  SIZE_T BlkRealSize;
  const char *BlkAllocFileName;
  int BlkAllocLineNumber;
  BOOL BlkValid;
#endif
  BYTE BlkHead[16];
} BLOCK_DESC, *PBLOCK_DESC;

typedef ULONG (HEAPREGPARM *PFNHEAP_WALK) __P((
  HEAP Heap,
  PBLOCK_DESC MemBlock,
  PVOID UserData
));

EXTERNAPI VOID HEAPAPI RtHeapWalk __P((
  HEAP Heap,
  ULONG Flags,
  PFNHEAP_WALK WalkFn,
  PVOID UserData
));

EXTERNAPI VOID HEAPAPI RtHeapUsageDump __P((
  HEAP Heap
));

enum {
  RtCallHandler          =   2,
  RtGenerateSignal       =   4,
  RtZeroMemory           =   8,
  /* for RtReAllocEx only */
  RtInplaceOnly          =  16,
  /* for RtReAllocEx and RtFreeEx*/
  RtNoDecommit           =  32,
  RtDiscardBlock         = 256,
};

/* alloc Size Bytes in the current thread Heap */
EXTERNAPI PVOID HEAPAPI RtAllocEx __P((
  ULONG Flags,
  SIZE_T Size,
  ULONG Align
));

EXTERNAPI PVOID HEAPAPI RtAllocExDbg __P((
  ULONG Flags,
  SIZE_T Size,
  ULONG Align,
  const char *FileName,
  int LineNumber
));

EXTERNAPI VOID HEAPAPI RtFreeEx __P((
  ULONG Flags,
  PVOID MemAddr
));

EXTERNAPI VOID HEAPAPI RtFreeExDbg __P((
  ULONG Flags,
  PVOID MemAddr
));

EXTERNAPI PVOID HEAPAPI RtReAllocEx __P((
  ULONG Flags,
  PVOID MemAddr,
  SIZE_T Size,
  ULONG Align
));

EXTERNAPI PVOID HEAPAPI RtReAllocExDbg __P((
  ULONG Flags,
  PVOID MemAddr,
  SIZE_T Size,
  ULONG Align,
  const char *FileName,
  int LineNumber
));

EXTERNAPI SIZE_T HEAPAPI RtCompact __P((
  ULONG Flags,
  SIZE_T TrimSize
));

/* return Size in Bytes if block was allocated in the current thread or 0 */
EXTERNAPI SIZE_T HEAPAPI RtUsableSizeEx __P((
  PVOID MemAddr
));

EXTERNAPI SIZE_T HEAPAPI RtUsableSizeExDbg __P((
  PVOID MemAddr
));

EXTERNAPI VOID HEAPAPI RtUsageDump __P((
  VOID
));

EXTERNAPI PVOID malloc __P((
  SIZE_T Size
));

EXTERNAPI PVOID malloc_dbg __P((
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI PVOID calloc __P((
  SIZE_T Count,
  SIZE_T Size
));

EXTERNAPI PVOID calloc_dbg __P((
  SIZE_T Count,
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI PVOID memalign __P((
  SIZE_T Align,
  SIZE_T Size
));

EXTERNAPI PVOID memalign_dbg __P((
  SIZE_T Align,
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI PVOID valloc __P((
  SIZE_T Size
));

EXTERNAPI PVOID valloc_dbg __P((
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI PVOID pvalloc __P((
  SIZE_T Size
));

EXTERNAPI PVOID pvalloc_dbg __P((
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI VOID free __P((
  PVOID MemAddr
));

EXTERNAPI VOID free_dbg __P((
  PVOID MemAddr
));

EXTERNAPI PVOID realloc __P((
  PVOID MemAddr,
  SIZE_T Size
));

EXTERNAPI PVOID realloc_dbg __P((
  PVOID MemAddr,
  SIZE_T Size,
  const char *FileName,
  int LineNumber
));

EXTERNAPI int malloc_trim __P((
  SIZE_T Size
));

EXTERNAPI SIZE_T malloc_usable_size __P((
  PVOID MemAddr
));

EXTERNAPI SIZE_T malloc_usable_size_dbg __P((
  PVOID MemAddr
));

#ifndef NDEBUG
#define RtHeapAlloc(Heap, Flags, Size, Align) \
  RtHeapAllocDbg (Heap, Flags, Size, Align, __FILE__, __LINE__)
#define RtHeapFree(Heap, MemAddr) \
  RtHeapFreeDbg (Heap, MemAddr)
#define RtHeapReAlloc(Heap, Flags, MemAddr, Size, Align) \
  RtHeapReAllocDbg (Heap, Flags, MemAddr, Size, Align, __FILE__, __LINE__)
#define RtHeapUsableSize(Heap, MemAddr) \
  RtHeapUsableSizeDbg (Heap, MemAddr)
#define RtAllocEx(Flags, Size, Align) \
  RtAllocExDbg (Flags, Size, Align, __FILE__, __LINE__)
#define RtFreeEx(Flags, MemAddr) \
  RtFreeExDbg (Flags, MemAddr)
#define RtReAllocEx(Flags, MemAddr, Size, Align) \
  RtReAllocExDbg (Flags, MemAddr, Size, Align, __FILE__, __LINE__)
#define RtUsableSizeEx(MemAddr) \
  RtUsableSizeExDbg (MemAddr)

#define malloc(Size) \
  malloc_dbg (Size, __FILE__, __LINE__)
#define calloc(Count, Size) \
  calloc_dbg (Count, Size, __FILE__, __LINE__)
#define memalign(Align, Size) \
  memalign_dbg (Align, Size, __FILE__, __LINE__)
#define valloc(Size) \
  valloc_dbg (Size, __FILE__, __LINE__)
#define pvalloc(Size) \
  pvalloc_dbg (Size, __FILE__, __LINE__)
#define free(MemAddr) \
  free_dbg (MemAddr)
#define realloc(MemAddr, Size) \
  realloc_dbg (MemAddr, Size, __FILE__, __LINE__)
#define malloc_usable_size(MemAddr) \
  malloc_usable_size_dbg (MemAddr)

#else /* NDEBUG */
#define RtHeapAllocDbg(Heap, Flags, Size, Align, FileName, LineNumber) \
  RtHeapAlloc (Heap, Flags, Size, Align)
#define RtHeapFreeDbg(Heap, MemAddr) \
  RtHeapFree (Heap, MemAddr)
#define RtHeapReAllocDbg(Heap, Flags, MemAddr, Size, Align, FileName, LineNumber) \
  RtHeapReAlloc (Heap, Flags, MemAddr, Size, Align)
#define RtHeapUsableSizeDbg(Heap, MemAddr) \
  RtHeapUsableSize (Heap, MemAddr)
#define RtAllocExDbg(Flags, Size, Align, FileName, LineNumber) \
  RtAllocEx (Flags, Size, Align)
#define RtFreeExDbg(Flags, MemAddr) \
  RtFreeEx (Flags, MemAddr)
#define RtReAllocExDbg(Flags, MemAddr, Size, Align, FileName, LineNumber) \
  RtReAllocEx (Flags, MemAddr, Size, Align)
#define RtUsableSizeExDbg(MemAddr) \
  RtUsableSizeEx (MemAddr)

#define malloc_dbg(Size, FileName, LineNumber) \
  malloc (Size)
#define calloc_dbg(Count, Size, FileName, LineNumber) \
  calloc (Count, Size)
#define memalign_dbg(Align, Size, FileName, LineNumber) \
  memalign (Align, Size)
#define valloc_dbg(Size, FileName, LineNumber) \
  valloc (Size)
#define pvalloc_dbg(Size, FileName, LineNumber) \
  pvalloc (Size)
#define free_dbg(MemAddr) \
  free (MemAddr)
#define realloc_dbg(MemAddr, Size, FileName, LineNumber) \
  realloc (MemAddr, Size)
#define malloc_usable_size_dbg(MemAddr) \
  malloc_usable_size (MemAddr)

#endif /* NDEBUG */

#ifdef RPC_API_allocate_SUPPORT

__EXTERN_INLINE void __RPC_FAR * __RPC_API midl_user_allocate (SIZE_T Bytes) 
{ 
  return malloc (Bytes);
}

__EXTERN_INLINE void __RPC_USER midl_user_free (VOID __RPC_FAR *MemAddr)
{
  free (MemAddr);
}

#endif

__END_DECLS

#pragma pack (pop)

#endif /* __PvtHeap_h__ */
