/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _LIBC_LIMITS_
#define _LIBC_LIMITS_

#include <features.h>

/*Maximum number of I/O operations in a single list I/O call
supported by the implementation.*/

#define AIO_LISTIO_MAX _POSIX_AIO_LISTIO_MAX

/*Maximum number of outstanding asynchronous I/O operations
supported by the implementation.*/

#define AIO_MAX _POSIX_AIO_MAX

/*The maximum amount by which a process can decrease its
asynchronous I/O priority level from its own scheduling priority.*/

#define AIO_PRIO_DELTA_MAX 0

/*Maximum length of argument to the exec functions
including environment data.*/

#define ARG_MAX _POSIX_ARG_MAX

/*Maximum number of functions that may be registered with atexit().*/
#define ATEXIT_MAX 512

/*Maximum number of simultaneous processes per real user ID.*/
#define CHILD_MAX 512

/*Maximum number of timer expiration overruns.*/
#define DELAYTIMER_MAX _POSIX_DELAYTIMER_MAX

/*Maximum number of iovec structures that one
process has available for use with readv() or writev().*/
#ifndef IOV_MAX
# define IOV_MAX _XOPEN_IOV_MAX
#endif

/*Maximum length of a login name.*/
#define LOGIN_NAME_MAX _POSIX_LOGIN_NAME_MAX

/*The maximum number of open message queue descriptors a process may hold.*/
#define MQ_OPEN_MAX _POSIX_MQ_OPEN_MAX

/*The maximum number of message priorities supported by the implementation.*/
#define MQ_PRIO_MAX _POSIX_MQ_PRIO_MAX

/*Maximum number of files that one process can have open at any one time.*/
#define OPEN_MAX 512

/*Size in bytes of a page. */
#define PAGESIZE 4096

/*Same as PAGESIZE. If either PAGESIZE or PAGE_SIZE is defined, the other will be
defined with the same value.*/
#define PAGE_SIZE PAGESIZE

/*Maximum number of significant bytes in a password (not
including terminating null).*/
#define PASS_MAX 256

/*Maximum number of attempts made to destroy a thread's
thread-specific data values on thread exit.*/
#define PTHREAD_DESTRUCTOR_ITERATIONS _POSIX_THREAD_DESTRUCTOR_ITERATIONS

/*Maximum number of data keys that can be created by a process.*/
#define PTHREAD_KEYS_MAX _POSIX_THREAD_KEYS_MAX

/*Minimum size in bytes of thread stack storage.*/
#define PTHREAD_STACK_MIN 32768

/*Maximum number of threads that that can be created per process.*/
#define PTHREAD_THREADS_MAX _POSIX_THREAD_THREADS_MAX

/*Maximum number of realtime signals reserved for application use in this
implementation.*/
#define RTSIG_MAX _POSIX_RTSIG_MAX

/*Maximum number of semaphores that a process may have.*/
#define SEM_NSEMS_MAX _POSIX_SEM_NSEMS_MAX

/*The maximum value a semaphore may have.*/
#define SEM_VALUE_MAX _POSIX_SEM_VALUE_MAX

/*Maximum number of queued signals that a process
may send and have pending at the receiver(s) at any time.*/
#define SIGQUEUE_MAX _POSIX_SIGQUEUE_MAX

/*The number of streams that one process can have open at
one time. If defined, it has the same value as {FOPEN_MAX} (see
<stdio.h>).*/
#define STREAM_MAX _POSIX_STREAM_MAX

/*Maximum number of timers per-process supported by the implementation.*/
#define TIMER_MAX _POSIX_TIMER_MAX

/*Maximum length of terminal device name.*/
#define TTY_NAME_MAX _POSIX_TTY_NAME_MAX

/*Maximum number of bytes supported for the name of a time zone (not of the TZ variable).
The values in the following list may be constants within an
implementation or may vary from one pathname to another. For example,
file systems or directories may have different characteristics.
A definition of one of the values will be omitted from the
<limits.h> header on specific implementations where the corresponding value is
equal to or greater than the stated minimum, but where the value can vary
depending on the file to which it is applied. The actual value
supported for a specific pathname will be provided by the
pathconf() function.*/

#define TZNAME_MAX _POSIX_TZNAME_MAX

/*Minimum number of bits needed to represent, as a signed integer
value, the maximum size of a regular file allowed in the specified 
directory.*/
#ifdef __USE_LARGEFILE64
# define FILESIZEBITS 64
#else
# define FILESIZEBITS 32
#endif

/*Maximum number of links to a single file.*/
#define LINK_MAX

/*Maximum number of bytes in a terminal canonical input line.*/
#define MAX_CANON _POSIX_MAX_CANON

/*Minimum number of bytes for which space will be available in a
terminal input queue; therefore, the maximum number of bytes a
portable application may require to be typed as input before reading them.*/
#define MAX_INPUT _POSIX_MAX_INPUT

/*Maximum number of bytes in a filename (not including terminating null).*/
#define NAME_MAX _POSIX_NAME_MAX

/*Maximum number of bytes in a pathname, including the terminating null character.*/
#define PATH_MAX _POSIX_PATH_MAX

/*Maximum number of bytes that is guaranteed to be atomic when
writing to a pipe. The magnitude limitations in the following list will be fixed by
specific implementations. An application should assume that the value supplied by
<limits.h> in a specific implementation is the minimum that pertains whenever the
application is run under that implementation. A specific instance of a
specific implementation may increase the value relative to that supplied by
<limits.h> for that implementation. The actual value supported by a specific
instance will be provided by the sysconf() function.*/

#define PIPE_BUF _POSIX_PIPE_BUF

/*Maximum obase values allowed by the bc utility.*/
#define BC_BASE_MAX _POSIX2_BC_BASE_MAX

/*Maximum number of elements permitted in an array by the bc utility.*/
#define BC_DIM_MAX _POSIX2_BC_DIM_MAX

/*Maximum scale value allowed by the bc utility.*/
#define BC_SCALE_MAX _POSIX2_BC_SCALE_MAX

/*Maximum length of a string constant accepted by the bc utility.*/
#define BC_STRING_MAX _POSIX2_BC_STRING_MAX

/*Maximum number of weights that can be assigned to an entry of the
LC_COLLATE order keyword in the locale definition file;
see the XBD specification, Locale.*/
#define COLL_WEIGHTS_MAX _POSIX2_COLL_WEIGHTS_MAX

/*Maximum number of expressions that can be nested within parentheses by the expr utility.*/
#define EXPR_NEST_MAX _POSIX2_EXPR_NEST_MAX

/*Unless otherwise noted, the maximum length, in bytes, of a utility's input
line (either standard input or another file), when the utility is described
as processing text files.  The length includes room for the trailing newline.*/
#define LINE_MAX _POSIX2_LINE_MAX

/*Maximum number of simultaneous supplementary group IDs per process.*/
#define NGROUPS_MAX 16

/*Maximum number of repeated occurrences of a regular expression
permitted when using the interval notation \{m,n\};
see the XBD specification, Regular Expressions.*/
#define RE_DUP_MAX _POSIX2_RE_DUP_MAX

/*These are symbolic names for the most
restrictive value for certain features on a system supporting the
Realtime Feature Group.  A conforming implementation will provide
values no larger than these values.  A portable application will not
require a smaller value for correct operation.
The CLOCK_REALTIME clock resolution, in nanoseconds*/
#define _POSIX_CLOCKRES_MIN 20 000 000

/*These are symbolic names for the most restrictive value
for certain features on a system conforming to this specification.  Related
symbolic constants are defined elsewhere in this specification which
reflect the actual implementation and which need not be as restrictive.
A conforming implementation will provide values at least this large.
A portable application must not require a larger value for correct operation.
The number of I/O operations that can be specified in a
list I/O call.*/
#define _POSIX_AIO_LISTIO_MAX 2

/*The number of outstanding asynchronous I/O operations.*/
#define _POSIX_AIO_MAX 1

/*Maximum length of argument to the exec functions including
environment data.*/
#define _POSIX_ARG_MAX 4096

/*Maximum number of simultaneous processes per real user ID.*/
#define _POSIX_CHILD_MAX CHILD_MAX

/*The number of timer expiration overruns.*/
#define _POSIX_DELAYTIMER_MAX 32

/*Maximum number of links to a single file.*/
#define _POSIX_LINK_MAX 8

/*The size of the storage required for a login name, in bytes,
including the terminating null.*/
#define _POSIX_LOGIN_NAME_MAX 256

/*Maximum number of bytes in a terminal canonical input queue.*/
#define _POSIX_MAX_CANON 256

/*Maximum number of bytes allowed in a terminal input queue.*/
#define _POSIX_MAX_INPUT 256

/*The number of message queues that can be open for a single process.*/
#define _POSIX_MQ_OPEN_MAX 512

/*The maximum number of message priorities supported by the implementation.*/
#define _POSIX_MQ_PRIO_MAX 256

/*Maximum number of bytes in a filename (not including terminating null).*/
#define _POSIX_NAME_MAX 256

/*Maximum number of simultaneous supplementary group IDs per process.*/
#define _POSIX_NGROUPS_MAX NGROUPS_MAX

/*Maximum number of files that one process can have open at any one time.*/
#define _POSIX_OPEN_MAX OPEN_MAX

/*Maximum number of bytes in a pathname.*/
#define _POSIX_PATH_MAX 32000

/*Maximum number of bytes that is guaranteed to be atomic when writing to a pipe.*/
#define _POSIX_PIPE_BUF 512

/*The number of realtime signal numbers reserved for application use.*/
#define _POSIX_RTSIG_MAX 975

/*The number of semaphores that a process may have.*/
#define _POSIX_SEM_NSEMS_MAX 4096

/*The maximum value a semaphore may have.*/
#define _POSIX_SEM_VALUE_MAX ((int) ((~0u) >> 1))

/*The number of queued signals that a process may send and have pending
at the receiver(s) at any time.*/
#define _POSIX_SIGQUEUE_MAX 1024

/*The value that can be stored in an object of type ssize_t.*/
#define _POSIX_SSIZE_MAX ULONG_MAX

/*The number of streams that one process can have open at one time.*/
#define _POSIX_STREAM_MAX 512

/*The number of attempts made to destroy a thread's thread-specific data
values on thread exit.*/
#define _POSIX_THREAD_DESTRUCTOR_ITERATIONS 4

/*The number of data keys per process.*/
#define _POSIX_THREAD_KEYS_MAX 512

/*The number of threads per process.*/
#define _POSIX_THREAD_THREADS_MAX 4096

/*The per process number of timers.*/
#define _POSIX_TIMER_MAX 4096

/*The size of the storage required for a terminal device name, in bytes,
including the terminating null.*/
#define _POSIX_TTY_NAME_MAX 9

/*Maximum number of bytes supported for the name of a time zone (not of TZ variable).*/
#define _POSIX_TZNAME_MAX 256

/*Maximum obase values allowed by the bc utility.*/
#define _POSIX2_BC_BASE_MAX 99

/*Maximum number of elements permitted in an array by the bc utility.*/
#define _POSIX2_BC_DIM_MAX 2048

/*Maximum scale value allowed by the bc utility.*/
#define _POSIX2_BC_SCALE_MAX 99

/*Maximum length of a string constant accepted by the bc utility.*/
#define _POSIX2_BC_STRING_MAX 1000

/*Maximum number of weights that can be assigned to an entry of
the LC_COLLATE order keyword in the locale definition file;
see the XBD specification, Locale.*/
#define _POSIX2_COLL_WEIGHTS_MAX 2

/*Maximum number of expressions that can be nested within parentheses by the
expr utility.*/
#define _POSIX2_EXPR_NEST_MAX 32

/*Unless otherwise noted, the maximum length, in bytes, of a utility's input
line (either standard input or another file), when the utility is described
as processing text files. The length includes room for the trailing newline.*/
#define _POSIX2_LINE_MAX 2048

/*Maximum number of repeated occurrences of a regular expression permitted
when using the interval notation \{m,n\};
see the XBD specification, Regular Expressions.*/
#define _POSIX2_RE_DUP_MAX 255

/*Maximum number of iovec structures that one process has available
for use with readv() or writev().*/
#define _XOPEN_IOV_MAX _POSIX_UIO_MAXIOV

/*Maximum number of elements in an `iovec' array.*/
#define	_POSIX_UIO_MAXIOV	16

/*Number of bits in a type char.*/
#define CHAR_BIT  8

/*Maximum value of a type char.*/
#define CHAR_MAX SCHAR_MAX

/*Digits of precision of a type double.*/
#ifndef DBL_DIG
# define DBL_DIG 15
#endif

/*Maximum value of a type double.*/
#ifndef DBL_MAX
# define DBL_MAX 1.7976931348623158e + 308
#endif

/*Digits of precision of a type float.*/
#ifndef FLT_DIG
# define FLT_DIG 6
#endif

/*Maximum value of a float.*/
#ifndef FLT_MAX
# define FLT_MAX 3.402823466e + 38F
#endif

/*Maximum value of an int.*/
#define INT_MAX 2147483647

/*Number of bits in a long int.*/
#define LONG_BIT 32

/*Maximum value of a long int.*/
#define LONG_MAX INT_MAX

/*Maximum number of bytes in a character, for any supported locale.*/
#define MB_LEN_MAX 2

/*Maximum value of a type signed char.*/
#define SCHAR_MAX  127

/*Maximum value of a type short.*/
#define SHRT_MAX 32767

/*Maximum value of an object of type ssize_t.*/
#ifndef SSIZE_MAX
# define SSIZE_MAX _POSIX_SSIZE_MAX
#endif

/*Maximum value of a type unsigned char.*/
#define UCHAR_MAX 255

/*Maximum value of a type unsigned int.*/
#define UINT_MAX 0xFFFFFFFF

/*Maximum value of a type unsigned long int.*/
#define ULONG_MAX 0xFFFFFFFF

/*Maximum value for a type unsigned short int.*/
#define USHRT_MAX 0xFFFF

/*Number of bits in a word or type int.*/
#define WORD_BIT 16

/*Minimum value of a type char.*/
#define CHAR_MIN SCHAR_MIN

/*Minimum value of a type int.*/
#define INT_MIN -2147483648

/*Minimum value of a type long int.*/
#define LONG_MIN -2147483648

/*Minimum value of a type signed char.*/
#define SCHAR_MIN -128

/*Minimum value of a type short.*/
#define SHRT_MIN -32768

/*Maximum number of bytes in a character class name.*/
#define CHARCLASS_NAME_MAX 14

/*Maximum value of digit in calls to the printf() and scanf() functions.*/
#define NL_ARGMAX _POSIX_ARG_MAX

/*Maximum number of bytes in a LANG name.*/
#define NL_LANGMAX _POSIX2_LINE_MAX

/*Maximum message number.*/
#define NL_MSGMAX INT_MAX

/*Maximum number of bytes in an N-to-1 collation mapping.*/
#define NL_NMAX INT_MAX

/*Maximum set number.*/
#define NL_SETMAX INT_MAX

/*Maximum number of bytes in a message string.*/
#define NL_TEXTMAX INT_MAX

/*Default process priority. In the NT enviroment main
thread priority (when used raised priority).*/
#define NZERO 8

/*Minimum number of unique pathnames generated by tmpnam().
Maximum number of times an application can call tmpnam() reliably.*/
#ifndef TMP_MAX
# define TMP_MAX 10000
#endif

#endif /*limits.h*/