/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _MALLOC_
#define _MALLOC_

#include <features.h>
#define	__need_NULL
#define	__need_size_t
#include <stddef.h>

__BEGIN_DECLS

#pragma pack (push, 4)

/* Allocate SIZE bytes of memory.  */
EXTERNAPI void *malloc __P ((size_t __size));
EXTERNAPI void *malloc_dbg __P ((size_t __size, const char *__file, int __line));

/* Allocate NMEMB elements of SIZE bytes each, all initialized to 0.  */
EXTERNAPI void *calloc __P ((size_t __nmemb, size_t __size));
EXTERNAPI void *calloc_dbg __P ((size_t __nmemb, size_t __size,
                                const char *__file, int __line));

/* Re-allocate the previously allocated block in __ptr, making the new
   block SIZE bytes long.  */
EXTERNAPI void *realloc __P ((void *__ptr, size_t __size));
EXTERNAPI void *realloc_dbg __P ((void *__ptr, size_t __size,
                                 const char *__file, int __line));

/* Free a early allocated block by any `?alloc?' function.  */
EXTERNAPI void free __P ((void *__ptr));
EXTERNAPI void free_dbg __P ((void *__ptr));
EXTERNAPI void cfree __P ((void *__ptr));
EXTERNAPI void cfree_dbg __P ((void *__ptr));

/* Allocate SIZE bytes allocated to ALIGNMENT bytes.  */
EXTERNAPI void *memalign __P ((size_t __alignment, size_t __size));
EXTERNAPI void *memalign_dbg __P ((size_t __alignment, size_t __size,
                                  const char *__file, int __line));

/* Allocate SIZE bytes on a page boundary.  */
EXTERNAPI void *valloc __P ((size_t __size));
EXTERNAPI void *valloc_dbg __P ((size_t __size, const char *__file, int __line));

/* Equivalent to valloc(minimum-page-that-holds(n)), that is, round up
   __size to nearest pagesize. */
EXTERNAPI void * pvalloc __P ((size_t __size));
EXTERNAPI void * pvalloc_dbg __P ((size_t __size, const char *__file, int __line));

/* SVID2/XPG mallinfo structure */
struct mallinfo {
  int arena;    /* total space allocated from system */
  int ordblks;  /* unused -- always zero */
  int smblks;   /* unused -- always zero */
  int hblks;    /* number of mmapped regions */
  int hblkhd;   /* total space in mmapped regions */
  int usmblks;  /* unused -- always zero */
  int fsmblks;  /* unused -- always zero */
  int uordblks; /* total allocated space */
  int fordblks; /* total non-inuse space */
  int keepcost; /* unused -- always zero */
};

/* Returns a copy of the updated current mallinfo. */
EXTERNAPI struct mallinfo mallinfo __P ((void));

/* SVID2/XPG mallopt options */
#ifndef M_MXFAST
# define M_MXFAST  1	/* UNUSED in this malloc */
#endif
#ifndef M_NLBLKS
# define M_NLBLKS  2	/* UNUSED in this malloc */
#endif
#ifndef M_GRAIN
# define M_GRAIN   3	/* UNUSED in this malloc */
#endif
#ifndef M_KEEP
# define M_KEEP    4	/* UNUSED in this malloc */
#endif

/* mallopt options that actually do something */
#define M_TRIM_THRESHOLD    -1
#define M_TOP_PAD           -2
#define M_MMAP_THRESHOLD    -3
#define M_MMAP_MAX          -4
#define M_CHECK_ACTION      -5

/* General SVID/XPG interface to tunable parameters. */
EXTERNAPI int mallopt __P ((int __param, int __val));

/* Release all but __pad bytes of freed top-most memory back to the
   system. Return 1 if successful, else 0. */
EXTERNAPI int malloc_trim __P ((size_t __pad));

/* Report the number of usable allocated bytes associated with allocated
   chunk __ptr. */
EXTERNAPI size_t malloc_usable_size __P ((void *__ptr));
EXTERNAPI size_t malloc_usable_size_dbg __P ((void *__ptr));

/* Prints brief summary statistics on stderr. */
EXTERNAPI void malloc_stats __P ((void));

/* Record the state of all malloc variables in an opaque data structure. */
EXTERNAPI void *malloc_get_state __P ((void));

/* Restore the state of all malloc variables from data obtained with
   malloc_get_state(). */
EXTERNAPI int malloc_set_state __P ((void *__ptr));

#ifndef NDEBUG
# define malloc(__size) malloc_dbg (__size, __FILE__, __LINE__)
# define calloc(Count, __size) calloc_dbg (Count, __size, __FILE__, __LINE__)
# define memalign(Align, __size) memalign_dbg (Align, __size, __FILE__, __LINE__)
# define valloc(__size) valloc_dbg (__size, __FILE__, __LINE__)
# define pvalloc(__size) pvalloc_dbg (__size, __FILE__, __LINE__)
# define free(__ptr) free_dbg (__ptr)
# define cfree(__ptr) cfree_dbg (__ptr)
# define realloc(__ptr, __size) realloc_dbg (__ptr, __size, __FILE__, __LINE__)
# define malloc_usable_size(__ptr) malloc_usable_size_dbg (__ptr)
#else /* NDEBUG */
# define malloc_dbg(__size, __file, __line) malloc (__size)
# define calloc_dbg(Count, __size, __file, __line) calloc (Count, __size)
# define memalign_dbg(Align, __size, __file, __line) memalign (Align, __size)
# define valloc_dbg(__size, __file, __line) valloc (__size)
# define pvalloc_dbg(__size, __file, __line) pvalloc (__size)
# define free_dbg(__ptr) free (__ptr)
# define cfree_dbg(__ptr) cfree (__ptr)
# define realloc_dbg(__ptr, __size, __file, __line) realloc (__ptr, __size)
# define malloc_usable_size_dbg(__ptr) malloc_usable_size (__ptr)
#endif /* NDEBUG */

#ifdef RPC_API_allocate_SUPPORT

__EXTERN_INLINE void __RPC_FAR * __RPC_API midl_user_allocate (size_t __size) 
{ 
  return malloc (__size);
}

__EXTERN_INLINE void __RPC_USER midl_user_free (void __RPC_FAR *__ptr)
{
  free (__ptr);
}

#endif

__END_DECLS

#pragma pack (pop)

#endif /* malloc.h */
