/*
 * X86Math Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov   All Rights Reserved.
 *
 * The Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * The Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the GNU C Library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA.  */

/*
 *  ISO C Standard: 4.5 MATHEMATICS <math.h>
 */

#ifndef _MATH_
#define _MATH_

#include <features.h>

__BEGIN_DECLS

#ifndef __GNUC__
# define __NO_LONG_DOUBLE_MATH
#endif

/* Get machine-dependent HUGE_VAL value (returned on overflow).
   On all IEEE754 machines, this is +Infinity.  */
#include <bits/huge.h>

/* Get machine-dependent NAN value (returned for some domain errors).  */
#ifdef   __USE_UNIX98
# include <bits/nan.h>
/* Get general and ISO C 9X specific information.  */
# include <bits/mathdef.h>

#endif

#if defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98
/* This variable is used by `gamma' and `lgamma'.  */
EXTERNAPI int *__signgamp __P((void));
#define signgam (*__signgamp ())
#endif

#ifndef __NO_MATH_INLINES_EXT

#if defined __USE_MISC || defined __USE_UNIX98

__EXTERN_INLINE float copysignf (float x, float y) __THROW {
  unsigned int __uf = ((*(unsigned*)&x) & 0x7fffffff) | ((*(unsigned*)&y) & 0x80000000);
  return *(float*)&__uf;
}
__EXTERN_INLINE double copysign (double x, double y) __THROW {
  unsigned int __uv[2];
  __uv[0] = ((unsigned*)&x)[0];
  __uv[1] = ((((unsigned*)&x)[1]) & 0x7fffffff) | ((((unsigned*)&y)[1]) & 0x80000000);
  return (*(double*)__uv);
}
# ifndef __NO_LONG_DOUBLE_MATH
__EXTERN_INLINE long double copysignl (long double x, long double y) __THROW {
  unsigned int __uv[3];
  __uv[0] = ((unsigned*)&x)[0];
  __uv[1] = ((unsigned*)&x)[1];
  __uv[2] = ((((unsigned*)&x)[2]) & 0x7fff) | ((((unsigned*)&y)[2]) & 0x8000);
  return (*(long double*)__uv);
}
# endif

#endif //defined __USE_MISC || defined __USE_UNIX98

#if defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98
extern float lgammaf_r __P ((float x, int *signgamp));
extern double lgamma_r __P ((double x, int *signgamp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double lgammal_r __P ((long double x, int *signgamp));
# endif

__EXTERN_INLINE float lgammaf (float x) __THROW {
  int sign;
  register float val = lgammaf_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
__EXTERN_INLINE double lgamma (double x) __THROW {
  int sign;
  register double val = lgamma_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
# ifndef __NO_LONG_DOUBLE_MATH
__EXTERN_INLINE long double lgammal (long double x) __THROW {
  int sign;
  register long double val = lgammal_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
# endif

__EXTERN_INLINE float gammaf (float x) __THROW {
  int sign;
  register float val = lgammaf_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
__EXTERN_INLINE double gamma (double x) __THROW {
  int sign;
  register double val = lgamma_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
# ifndef __NO_LONG_DOUBLE_MATH
__EXTERN_INLINE long double gammal (long double x) __THROW {
  int sign;
  register long double val = lgammal_r (x, &sign);
#ifdef signgam
  signgam = sign;
#endif
  return val;
}
# endif

extern float gammaf_r __P ((float x, int *signgamp));
extern double gamma_r __P ((double x, int *signgamp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double gammal_r __P ((long double x, int *signgamp));
# endif

__EXTERN_INLINE float tgammaf (float x) __THROW {
  int sign;
  register float val = gammaf_r (x, &sign);
  if (sign < 0) val = -val;
  return val;
}
__EXTERN_INLINE double tgamma (double x) __THROW {
  int sign;
  register double val = gamma_r (x, &sign);
  if (sign < 0) val = -val;
  return val;
}
# ifndef __NO_LONG_DOUBLE_MATH
__EXTERN_INLINE long double tgammal (long double x) __THROW {
  int sign;
  register long double val = gammal_r (x, &sign);
  if (sign < 0) val = -val;
  return val;
}
# endif

#endif //defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98

#endif

extern int __fpclassifyf __P ((float x));
extern int fpclassifyf __P ((float x));
extern int __fpclassify __P ((double x));
extern int fpclassify __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int __fpclassifyl __P ((long double x));
extern int fpclassifyl __P ((long double x));
# else
__EXTERN_INLINE int __fpclassifyl (long double x) __THROW { return __fpclassify ((double)x); }
__EXTERN_INLINE int fpclassifyl (long double x) __THROW { return fpclassify ((double)x); }
# endif

extern int __signbitf __P ((float x));
extern int signbitf __P ((float x));
extern int __signbit __P ((double x));
extern int signbit __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int __signbitl __P ((long double x));
extern int signbitl __P ((long double x));
# else
__EXTERN_INLINE int __signbitl (long double x) __THROW { return __signbit ((double)x); }
__EXTERN_INLINE int signbitl (long double x) __THROW { return signbit ((double)x); }
# endif

extern int __finitef __P ((float x));
extern int __finite __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int __finitel __P ((long double x));
# else
__EXTERN_INLINE int __finitel (long double x) __THROW { return __finite ((double)x); }
# endif

extern int __isinff __P ((float x));
extern int __isinf __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int __isinfl __P ((long double x));
# else
__EXTERN_INLINE int __isinfl (long double x) __THROW { return __finite ((double)x); }
# endif

# ifdef __USE_MISC
extern int finitef __P ((float x));
extern int finite __P ((double x));
#  ifndef __NO_LONG_DOUBLE_MATH
extern int finitel __P ((long double x));
#  else
__EXTERN_INLINE int finitel (long double x) __THROW { return finite ((double)x); }
#  endif

extern int isinff __P ((float x));
extern int isinf __P ((double x));
#  ifndef __NO_LONG_DOUBLE_MATH
extern int isinfl __P ((long double x));
#  else
__EXTERN_INLINE int isinfl (long double x) __THROW { return isinf ((double)x); }
#  endif
# endif //__USE_MISC

extern int __isnanf __P ((float x));
extern int __isnan __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int __isnanl __P ((long double x));
# else
__EXTERN_INLINE int __isnanl (long double x) __THROW { return __isnan ((double)x); }
# endif

# if defined __USE_MISC || defined __USE_XOPEN
extern int isnanf __P ((float x));
extern int isnan __P ((double x));
#  ifndef __NO_LONG_DOUBLE_MATH
extern int isnanl __P ((long double x));
#  else
__EXTERN_INLINE int isnanl (long double x) __THROW { return isnan ((double)x); }
#  endif
# endif //defined __USE_MISC || defined __USE_XOPEN

/* ISO C 9X defines some generic macros which work on any data type.  */
#if __USE_UNIX98

/* Get the architecture specific values describing the floating-point
   evaluation.  The following symbols will get defined:

    float_t floating-point type at least as wide as `float' used
    to evaluate `float' expressions
    double_t  floating-point type at least as wide as `double' used
    to evaluate `double' expressions

    FLT_EVAL_METHOD
    Defined to
      0 if `float_t' is `float' and `double_t' is `double'
      1 if `float_t' and `double_t' are `double'
      2 if `float_t' and `double_t' are `long double'
      else  `float_t' and `double_t' are unspecified

    INFINITY  representation of the infinity value of type `float'

    FP_FAST_FMA
    FP_FAST_FMAF
    FP_FAST_FMAL
    If defined it indicates that the `fma' function
    generally executes about as fast as a multiply and an add.
    This macro is defined only iff the `fma' function is
    implemented directly with a hardware multiply-add instructions.

    FP_ILOGB0 Expands to a value returned by `ilogb (0.0)'.
    FP_ILOGBNAN Expands to a value returned by `ilogb (NAN)'.

    DECIMAL_DIG Number of decimal digits supported by conversion between
    decimal and all internal floating-point formats.

*/

/* All floating-point numbers can be put in one of these categories.  */
#define FP_NAN 1
#define FP_INFINITE 2
#define FP_ZERO 3
#define FP_SUBNORMAL 4
#define FP_NORMAL 5

#ifdef __GNUC__

# define fpclassify(x) \
  ({ register short __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ( \
     "fxam \n\t" \
     "fnstsw \n\t" \
     "sahf \n\t" \
     "jbe 2f \n\t" \
     "movw $5, %%ax \n\t" \
     "jmp 6f \n\t" \
     "2: jne 4f \n\t" \
     "jp 3f \n\t" \
     "movw $3, %%ax \n\t" \
     "jmp 6f \n\t" \
     "3: movw $4, %%ax \n\t" \
     "jmp 6f \n\t" \
     "4: jnp 5f \n\t" \
     "movw $2, %%ax \n\t" \
     "jmp 6f \n\t" \
     "5: movw $1, %%ax \n\t" \
     "6: \n\t" \
     : "=a" (__result) : "t" (__x)); \
     __result; })

     /*if ((__result & 0x4500) != 0x4500) { \
       if ((__result & 0x500) != 0x500) { \
         if ((__result & 0x100) == 0) { \
           if ((__result & 0x4000) == 0) \
             __result = FP_NORMAL; \
           else \
             __result = FP_ZERO; \
         } else \
           __result = FP_NAN; \
       } else \
         __result = FP_INFINITE; \
     } else \
       __result = FP_SUBNORMAL; \ */

# define signbit(x) \
  ({ register int __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ("fxam; fnstsw; andw $0x200, %%ax; shlw $6, %%ax" \
     : "=a" (__result) : "t" (__x)); \
     __result; })

# define isnormal(x) \
  ({ register short __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ("fxam; fnstsw" \
     : "=a" (__result) : "t" (__x)); \
     (__result & 0x4500) == 0x400; })

# define isnan(x) \
  ({ register short __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ("fxam; fnstsw" \
     : "=a" (__result) : "t" (__x)); \
     (__result & 0x500) == 0x100; })

# define isinf(x) \
  ({ register short __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ("fxam; fnstsw" \
     : "=a" (__result) : "t" (__x)); \
     (__result & 0x500) == 0x500; })

# define isfinite(x) \
  ({ register char __result; \
     register long double __x = (long double)x; \
     __asm__ __volatile__ ("fxam; fnstsw; sahf; setae %%al" \
     : "=a" (__result) : "t" (__x)); \
     __result; })

__EXTERN_INLINE int __fpclassifyf (float x) __THROW { return fpclassify (x); }
__EXTERN_INLINE int fpclassifyf (float x) __THROW { return fpclassify (x); }
__EXTERN_INLINE int __fpclassify (double x) __THROW { return fpclassify (x); }
__EXTERN_INLINE int __fpclassifyl (long double x) __THROW { return fpclassify (x); }
__EXTERN_INLINE int fpclassifyl (long double x) __THROW { return fpclassify (x); }
__EXTERN_INLINE int __signbitf (float x) __THROW { return signbit (x); }
__EXTERN_INLINE int signbitf (float x) __THROW { return signbit (x); }
__EXTERN_INLINE int __signbit (double x) __THROW { return signbit (x); }
__EXTERN_INLINE int __signbitl (long double x) __THROW { return signbit (x); }
__EXTERN_INLINE int signbitl (long double x) __THROW { return signbit (x); }

__EXTERN_INLINE int __isnanf (float x) __THROW { return isnan (x); }
__EXTERN_INLINE int __isnan (double x) __THROW { return isnan (x); }
__EXTERN_INLINE int __isnanl (long double x) __THROW { return isnan (x); }

__EXTERN_INLINE int isnanf (float x) __THROW { return isnan (x); }
__EXTERN_INLINE int isnanl (long double x) __THROW { return isnan (x); }

__EXTERN_INLINE int isinff (float x) __THROW { return isinf (x); }
__EXTERN_INLINE int isinfl (long double x) __THROW { return isinf (x); }

__EXTERN_INLINE int finitef (float x) __THROW { return isfinite (x); }
__EXTERN_INLINE int finite (double x) __THROW { return isfinite (x); }
__EXTERN_INLINE int finitel (long double x) __THROW { return isfinite (x); }

__EXTERN_INLINE int __isinff (float x) __THROW { return isinf (x); }
__EXTERN_INLINE int __isinf (double x) __THROW { return isinf (x); }
__EXTERN_INLINE int __isinfl (long double x) __THROW { return isinf (x); }

__EXTERN_INLINE int __finitef (float x) __THROW { return isfinite (x); }
__EXTERN_INLINE int __finite (double x) __THROW { return isfinite (x); }
__EXTERN_INLINE int __finitel (long double x) __THROW { return isfinite (x); }

#else

# ifndef __cplusplus
/* Return number of classification appropriate for X.  */
#  ifdef __NO_LONG_DOUBLE_MATH
#   define fpclassify(x) \
     (sizeof (x) == sizeof (float) ? __fpclassifyf (x) : __fpclassify (x))
#  else
#   define fpclassify(x) \
     (sizeof (x) == sizeof (float) ?                \
        __fpclassifyf (x)                 \
      : sizeof (x) == sizeof (double) ?               \
        __fpclassify (x) : __fpclassifyl (x))
#  endif

/* Return nonzero value if sign of X is negative.  */
#  ifdef __NO_LONG_DOUBLE_MATH
#   define signbit(x) \
     (sizeof (x) == sizeof (float) ? __signbitf (x) : __signbit (x))
#  else
#   define signbit(x) \
     (sizeof (x) == sizeof (float) ?                \
        __signbitf (x)                    \
      : sizeof (x) == sizeof (double) ?               \
        __signbit (x) : __signbitl (x))
#  endif

/* Return nonzero value if X is neither zero, subnormal, Inf, nor NaN.  */
#  define isnormal(x) (fpclassify (x) == FP_NORMAL)

/* Return nonzero value if X is a NaN.  We could use `fpclassify' but
   we already have this functions `__isnan' and it is faster.  */
#  ifdef __NO_LONG_DOUBLE_MATH
#   define isnan(x) \
     (sizeof (x) == sizeof (float) ? __isnanf (x) : __isnan (x))
#  else
#  define isnan(x) \
     (sizeof (x) == sizeof (float) ?                \
        __isnanf (x)                    \
      : sizeof (x) == sizeof (double) ?               \
        __isnan (x) : __isnanl (x))
#  endif

/* Return nonzero value is X is positive or negative infinity.  */
#  ifdef __NO_LONG_DOUBLE_MATH
#   define isinf(x) \
     (sizeof (x) == sizeof (float) ? __isinff (x) : __isinf (x))
#  else
#   define isinf(x) \
     (sizeof (x) == sizeof (float) ?                \
        __isinff (x)                    \
      : sizeof (x) == sizeof (double) ?               \
        __isinf (x) : __isinfl (x))
#  endif

/* Return nonzero value if X is not +-Inf or NaN.  */
#  ifdef __NO_LONG_DOUBLE_MATH
#   define isfinite(x) \
     (sizeof (x) == sizeof (float) ? __finitef (x) : __finite (x))
#  else
#   define isfinite(x) \
     (sizeof (x) == sizeof (float) ?                \
        __finitef (x)                   \
      : sizeof (x) == sizeof (double) ?               \
        __finite (x) : __finitel (x))
#  endif

# else /* __cplusplus */

static inline int fpclassify (float fe) __THROW { return __fpclassifyf (x); }
static inline int fpclassify (double fe) __THROW { return __fpclassify (x); }
#  ifdef __NO_LONG_DOUBLE_MATH
static inline int fpclassify (long double x) __THROW { return __fpclassify ((double)x); }
#  else
static inline int fpclassify (long double x) __THROW { return __fpclassifyl (x); }
#  endif
#define fpclassify fpclassify

static inline int signbit (float x)  __THROW { return ((short *)&(x))[1] & 0x8000; }
static inline int signbit (double x) __THROW { return ((short *)&(x))[3] & 0x8000; }
# ifdef __NO_LONG_DOUBLE_MATH
static inline int signbit (long double x) __THROW { double __x = (double)x; return ((short *)&(__x))[3] & 0x8000; }
# else
static inline int signbit (long double x) __THROW { return ((short *)&(x))[4] & 0x8000; }
# endif
#define signbit signbit

static inline int isnormal (float x) __THROW { return __fpclassifyf (x) == FP_NORMAL; }
static inline int isnormal (double x)  __THROW { return __fpclassify (x) == FP_NORMAL; }
# ifdef __NO_LONG_DOUBLE_MATH
static inline int isnormal (long double x) __THROW { return __fpclassify ((double)x) == FP_NORMAL; }
# else
static inline int isnormal (long double x) __THROW { return __fpclassifyl (x) == FP_NORMAL; }
# endif
#define isnormal isnormal

static inline int isnan (float x) __THROW { return __isnanf (x); }
static inline int isnan (double x) __THROW { return __isnan (x); }
# ifdef __NO_LONG_DOUBLE_MATH
static inline int isnan (long double x) __THROW { return __isnan ((double)x); }
# else
static inline int isnan (long double x) __THROW { return __isnanl (x);   }
# endif
#define isnan isnan

static inline int isfinite (float x) __THROW { return __finitef (x); }
static inline int isfinite (double x) __THROW { return __finite (x); }
# ifdef __NO_LONG_DOUBLE_MATH
static inline int isfinite (long double x) __THROW { return __finite ((double)x); }
# else
static inline int isfinite (long double x) __THROW { return __finitel (x); }
# endif
#define isfinite isfinite

static inline int isinf (float x) __THROW { return __isinff (x); }
static inline int isinf (double x) __THROW { return __isinf (x); }
# ifdef __NO_LONG_DOUBLE_MATH
static inline int isinf (long double x) __THROW { return __isinf ((double)x); }
# else
static inline int isinf (long double x) __THROW { return __isinfl (x); }
# endif
#define isinf isinf

# endif //__cplusplus

#endif /* !__GNUC__ */
#endif /* Use ISO C 9X.  */

#ifdef  __USE_MISC
/* Support for various different standard error handling behaviors.  */
typedef enum
{
  _IEEE_ = -1,  /* According to IEEE 754/IEEE 854.  */
  _SVID_, /* According to System V, release 4.  */
  _XOPEN_,  /* Nowadays also Unix98.  */
  _POSIX_,
  _ISOC_  /* Actually this is ISO C 9X.  */
} _LIB_VERSION_TYPE;

/* This variable can be changed at run-time to any of the values above to
   affect floating point error handling behavior (it may also be necessary
   to change the hardware FPU exception settings).  */
extern _LIB_VERSION_TYPE _LIB_VERSION;
#endif


#ifdef __USE_SVID
/* In SVID error handling, `matherr' is called with this description
   of the exceptional condition.

   We have a problem when using C++ since `exception' is a reserved
   name in C++.  */
# ifdef __cplusplus
struct __exception {
# else
struct exception {
# endif
  int    type;
  char  *name;
  double arg1;
  double arg2;
  double retval;
};

# ifdef __cplusplus
extern int matherr __P ((struct __exception *__exc));
# else
extern int matherr __P ((struct exception *__exc));
# endif

# define X_TLOSS  1.41484755040568800000e+16

/* Types of exceptions in the `type' field.  */
# define DOMAIN    1
# define SING      2
# define OVERFLOW  3
# define UNDERFLOW 4
# define TLOSS     5
# define PLOSS     6

/* SVID mode specifies returning this large value instead of infinity.  */
# define HUGE   FLT_MAX
# include <float.h>   /* Defines FLT_MAX.  */

#else /* !SVID */

# ifdef __USE_XOPEN
/* X/Open wants another strange constant.  */
#  define MAXFLOAT  FLT_MAX
#  include <float.h>
# endif

#endif  /* SVID */


/* Some useful constants.  */
#if defined __USE_BSD || defined __USE_XOPEN
# define M_E    2.7182818284590452354 /* e */
# define M_LOG2E  1.4426950408889634074 /* log_2 e */
# define M_LOG10E 0.43429448190325182765  /* log_10 e */
# define M_LN2    0.69314718055994530942  /* log_e 2 */
# define M_LN10   2.30258509299404568402  /* log_e 10 */
# define M_PI   3.14159265358979323846  /* pi */
# define M_PI_2   1.57079632679489661923  /* pi/2 */
# define M_PI_4   0.78539816339744830962  /* pi/4 */
# define M_1_PI   0.31830988618379067154  /* 1/pi */
# define M_2_PI   0.63661977236758134308  /* 2/pi */
# define M_2_SQRTPI 1.12837916709551257390  /* 2/sqrt(pi) */
# define M_SQRT2  1.41421356237309504880  /* sqrt(2) */
# define M_SQRT1_2  0.70710678118654752440  /* 1/sqrt(2) */
#endif

/* The above constants are not adequate for computation using `long double's.
   Therefore we provide as an extension constants with similar names as a
   GNU extension.  Provide enough digits for the 128-bit IEEE quad.  */
#ifdef __USE_GNU
# define M_El   2.7182818284590452353602874713526625L  /* e */
# define M_LOG2El 1.4426950408889634073599246810018922L  /* log_2 e */
# define M_LOG10El  0.4342944819032518276511289189166051L  /* log_10 e */
# define M_LN2l   0.6931471805599453094172321214581766L  /* log_e 2 */
# define M_LN10l  2.3025850929940456840179914546843642L  /* log_e 10 */
# define M_PIl    3.1415926535897932384626433832795029L  /* pi */
# define M_PI_2l  1.5707963267948966192313216916397514L  /* pi/2 */
# define M_PI_4l  0.7853981633974483096156608458198757L  /* pi/4 */
# define M_1_PIl  0.3183098861837906715377675267450287L  /* 1/pi */
# define M_2_PIl  0.6366197723675813430755350534900574L  /* 2/pi */
# define M_2_SQRTPIl  1.1283791670955125738961589031215452L  /* 2/sqrt(pi) */
# define M_SQRT2l 1.4142135623730950488016887242096981L  /* sqrt(2) */
# define M_SQRT1_2l 0.7071067811865475244008443621048490L  /* 1/sqrt(2) */
#endif

#ifdef __NO_LONG_DOUBLE_MATH

# define __mathfunc1(function) \
extern double function __P ((double __x)); \
extern float function##f __P ((float __x)); \
__EXTERN_INLINE long double function##l (long double __x) __THROW { return function ((double)__x); }

# define __mathfunc2(function) \
extern double function __P ((double __x, double __y)); \
extern float function##f __P ((float __x, float __y)); \
__EXTERN_INLINE long double function##l (__x, __y) __THROW { return function ((double)__x, (double)__y); }

#else

# define __mathfunc1(function) \
extern float function##f __P ((float __x)); \
extern double function __P ((double __x)); \
extern long double function##l __P ((long double __x));

# define __mathfunc2(function) \
extern float function##f __P ((float __x, float __y)); \
extern double function __P ((double __x, double __y)); \
extern long double function##l __P ((long double __x, long double __y));

#endif

__mathfunc1 (acos);
__mathfunc1 (acosh);
__mathfunc1 (asin);
__mathfunc1 (asinh);
__mathfunc1 (atan);
__mathfunc2 (atan2);
__mathfunc1 (atanh);

__mathfunc1 (cos);
__mathfunc1 (cosh);
__mathfunc1 (sin);
__mathfunc1 (sinh);
__mathfunc1 (tan);
__mathfunc1 (tanh);

#ifdef __USE_GNU
extern void sincosf __P ((float x, float *__sinx, float *__cosx));
extern void sincos __P ((double x, double *__sinx, double *__cosx));
# ifndef __NO_LONG_DOUBLE_MATH
extern void sincosl __P ((long double x, long double *__sinx, long double *__cosx));
# else
__EXTERN_INLINE void sincosl (long double x, long double *__sinx, long double *__cosx) __THROW
  { sincos ((double)x, (double*)__sinx, (double*)__cosx); }
# endif
#endif // __USE_GNU

__mathfunc1 (exp);
#ifdef __USE_GNU
__mathfunc1 (exp10);
__mathfunc1 (pow10);
__mathfunc1 (pow2);
#endif

#if defined __USE_MISC || defined __USE_XOPEN_EXTENDED || defined __USE_UNIX98
__mathfunc1 (expm1);
__mathfunc1 (log1p);
__mathfunc1 (logb);
#endif

#ifdef __USE_UNIX98
__mathfunc1 (exp2);
__mathfunc1 (log2);
#endif

extern float frexpf __P ((float x, int *expp));
extern double frexp __P ((double x, int *expp));
#ifndef __NO_LONG_DOUBLE_MATH
extern long double frexpl __P ((long double x, int *expp));
#else
__EXTERN_INLINE long double frexpl (long double x, int *expp) __THROW
  { return frexp ((double)x, expp); }
#endif
extern float ldexpf __P ((float x, int expp));
extern double ldexp __P ((double x, int expp));
#ifndef __NO_LONG_DOUBLE_MATH
extern long double ldexpl __P ((long double x, int expp));
#else
__EXTERN_INLINE long double ldexpl (long double x, int expp) __THROW
  { return ldexp ((double)x, expp); }
#endif

__mathfunc1 (log);
__mathfunc1 (log10);

extern float modff __P ((float x, float *iptr));
extern double modf __P ((double x, double *iptr));
#ifndef __NO_LONG_DOUBLE_MATH
extern long double modfl __P ((long double x, long double *iptr));
#else
__EXTERN_INLINE long double modfl (long double x, long double *iptr) __THROW
  { return modf ((double)x, (double*)iptr); }
#endif

#if defined __USE_MISC || defined __USE_XOPEN_EXTENDED || defined __USE_UNIX98
__mathfunc1 (cbrt);
#endif
__mathfunc1 (fabs);
#if defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98
__mathfunc2 (hypot);
#endif
__mathfunc2 (pow);
__mathfunc1 (sqrt);

__mathfunc1 (ceil);
__mathfunc1 (floor);

#if defined __USE_MISC || defined __USE_XOPEN_EXTENDED || defined __USE_UNIX98
__mathfunc1 (rint);
__mathfunc2 (nextafter);
# ifdef __USE_UNIX98
__mathfunc2 (nexttoward);
# endif
__mathfunc2 (remainder);
# if defined __USE_MISC || defined __USE_XOPEN_EXTENDED
__mathfunc2 (scalb);
# endif
extern float scalbnf __P ((float x, int expp));
extern double scalbn __P ((double x, int expp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double scalbnl __P ((long double x, int expp));
# else
__EXTERN_INLINE long double scalbnl (long double x, int expp) __THROW
  { return scalbn ((double)x, expp); }
# endif

extern int ilogbf __P ((float x));
extern int ilogb __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern int ilogbl __P ((long double x));
# else
__EXTERN_INLINE int ilogbl (long double x) __THROW
  { return ilogb ((double)x); }
# endif
#endif //defined __USE_MISC || defined __USE_XOPEN_EXTENDED || defined __USE_UNIX98

#ifdef __USE_UNIX98
extern float scalblnf __P ((float x, int expp));
extern double scalbln __P ((double x, int expp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double scalblnl __P ((long double x, int expp));
# else
__EXTERN_INLINE long double scalblnl (long double x, int expp) __THROW
  { return scalbln ((double)x, expp); }
# endif

__mathfunc1 (nearbyint);
__mathfunc1 (round);
__mathfunc1 (trunc);

extern float remquof __P ((float x, float y, int *quop));
extern double remquo __P ((double x, double y, int *quop));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double remquol __P ((long double x, long double y, int *quop));
# else
__EXTERN_INLINE long double remquol (long double x, long double y, int *quop) __THROW
  { return remquo ((double)x, (double)y, quop); }
# endif

extern long int lrintf __P ((float x));
extern long int lrint __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern long int lrintl __P ((long double x));
# else
__EXTERN_INLINE long int lrintl (long double x) __THROW
  { return lrint ((double)x); }
# endif
extern __int64 llrintf __P ((float x));
extern __int64 llrint __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern __int64 llrintl __P ((long double x));
# else
__EXTERN_INLINE __int64 llrintl (long double x) __THROW
  { return llrint ((double)x); }
# endif
extern long int lroundf __P ((float x));
extern long int lround __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern long int lroundl __P ((long double x));
# else
__EXTERN_INLINE long int lroundl (long double x) __THROW
  { return lround ((double)x); }
# endif
extern __int64 llroundf __P ((float x));
extern __int64 llround __P ((double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern __int64 llroundl __P ((long double x));
# else
__EXTERN_INLINE __int64 llroundl (long double x) __THROW
  { return llround ((double)x); }
# endif

__mathfunc2 (fdim);
__mathfunc2 (fmax);
__mathfunc2 (fmin);

extern float fmaf __P ((float x, float y, float z));
extern double fma __P ((double x, double y, double z));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double fmal __P ((long double x, long double y, long double z));
# else
__EXTERN_INLINE long double fmal (long double x, long double y, long double z) __THROW
  { return fma ((double)x, (double)y, (double)z); }
# endif
#endif //__USE_UNIX98

__mathfunc2 (fmod);

#ifdef __USE_MISC
__mathfunc2 (drem);
#endif

#if defined __USE_MISC || defined __USE_UNIX98
__mathfunc2 (copysign);
#endif //defined __USE_MISC || defined __USE_UNIX98

#if defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98
__mathfunc1 (erf);
__mathfunc1 (erfc);
__mathfunc1 (lgamma);
__mathfunc1 (tgamma);
#endif //defined __USE_MISC || defined __USE_XOPEN || defined __USE_UNIX98

#if defined __USE_MISC || defined __USE_XOPEN
__mathfunc1 (gamma);
#endif

#ifdef __USE_MISC
extern float lgammaf_r __P ((float x, int *signgamp));
extern double lgamma_r __P ((double x, int *signgamp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double lgammal_r __P ((long double x, int *signgamp));
# else
__EXTERN_INLINE long double lgammal_r (long double x, int *signgamp) __THROW
  { return lgamma_r ((double)x, signgamp); }
# endif

extern float gammaf_r __P ((float x, int *signgamp));
extern double gamma_r __P ((double x, int *signgamp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double gammal_r __P ((long double x, int *signgamp));
# else
__EXTERN_INLINE long double gammal_r (long double x, int *signgamp) __THROW
  { return gamma_r ((double)x, signgamp); }
# endif
#endif //__USE_MISC

#if defined __USE_MISC || defined __USE_XOPEN
__mathfunc1 (j0);
__mathfunc1 (y0);
__mathfunc1 (j1);
__mathfunc1 (y1);
extern float jnf __P ((int n, float x));
extern double jn __P ((int n, double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double jnl __P ((int n, long double x));
# else
__EXTERN_INLINE long double jnl (int n, long double x) __THROW
  { return jn (n, (double)x); }
# endif
extern float ynf __P ((int n, float x));
extern double yn __P ((int n, double x));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double ynl __P ((int n, long double x));
# else
__EXTERN_INLINE long double ynl (int n, long double x) __THROW
  { return yn (n, (double)x); }
# endif
#endif

#undef __mathfunc2
#undef __mathfunc1

#ifdef __USE_UNIX98
extern float nanf __P ((__const char *tagp));
extern double nan __P ((__const char *tagp));
# ifndef __NO_LONG_DOUBLE_MATH
extern long double nanl __P ((__const char *tagp));
# else
__EXTERN_INLINE long double nanl (__const char *tagp) __THROW { return nan (tagp); }
# endif
#endif //__USE_UNIX98

/* When compiling in strict ISO C compatible mode we must not use the
   inline functions since they, among other things, do not set the
   `errno' variable correctly.  */
#if defined __STRICT_ANSI__ && !defined __NO_MATH_INLINES
# define __NO_MATH_INLINES  1
#endif

/* Get machine-dependent inline versions (if there are any).  */
#ifdef __USE_EXTERN_INLINES
# include <bits/mathinline.h>
#endif

#if __USE_UNIX98
/* ISO C 9X defines some macros to compare number while taking care
   for unordered numbers.  Since many FPUs provide special
   instructions to support these operations and these tests are
   defined in <bits/mathinline.h>, we define the generic macros at
   this late point and only if they are not defined yet.  */
   
#ifndef __GNUC__

/* Return nonzero value if X is greater than Y.  */
# ifndef isgreater
#  define isgreater(__x, __y) (!isunordered ((__x), (__y)) && (__x) > (__y))
# endif

/* Return nonzero value if X is greater than or equal to Y.  */
# ifndef isgreaterequal
#  define isgreaterequal(__x, __y) (!isunordered ((__x), (__y)) && (__x) >= (__y))
# endif

/* Return nonzero value if X is less than Y.  */
# ifndef isless
#  define isless(x, y) (!isunordered ((__x), (__y)) && (__x) < (__y))
# endif

/* Return nonzero value if X is less than or equal to Y.  */
# ifndef islessequal
#  define islessequal(x, y) (!isunordered ((__x), (__y)) && (__x) <= (__y))
# endif

/* Return nonzero value if either X is less than Y or Y is less than X.  */
# ifndef islessgreater
#  define islessgreater(x, y) (!isunordered ((__x), (__y)) && ((__x) < (__y) || (__x) > (__y)))
# endif

/* Return nonzero value if arguments are unordered.  */
# ifndef isunordered
#  define isunordered(__u, __v) (fpclassify (__u) == FP_NAN || fpclassify (__v) == FP_NAN)
# endif

#endif

/* Return nonzero value if X is greater than Y.  */
# ifndef isgreater
#  define isgreater(x, y) \
  (__extension__                    \
   ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);           \
      !isunordered (__x, __y) && __x > __y; }))
# endif

/* Return nonzero value if X is greater than or equal to Y.  */
# ifndef isgreaterequal
#  define isgreaterequal(x, y) \
  (__extension__                    \
   ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);           \
      !isunordered (__x, __y) && __x >= __y; }))
# endif

/* Return nonzero value if X is less than Y.  */
# ifndef isless
#  define isless(x, y) \
  (__extension__                    \
   ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);           \
      !isunordered (__x, __y) && __x < __y; }))
# endif

/* Return nonzero value if X is less than or equal to Y.  */
# ifndef islessequal
#  define islessequal(x, y) \
  (__extension__                    \
   ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);           \
      !isunordered (__x, __y) && __x <= __y; }))
# endif

/* Return nonzero value if either X is less than Y or Y is less than X.  */
# ifndef islessgreater
#  define islessgreater(x, y) \
  (__extension__                    \
   ({ __typeof__(x) __x = (x); __typeof__(y) __y = (y);           \
      !isunordered (__x, __y) && (__x < __y || __y < __x); }))
# endif

/* Return nonzero value if arguments are unordered.  */
# ifndef isunordered
#  define isunordered(u, v) \
  (__extension__                    \
   ({ __typeof__(u) __u = (u); __typeof__(v) __v = (v);           \
      fpclassify (__u) == FP_NAN || fpclassify (__v) == FP_NAN; }))
# endif

#endif

__END_DECLS

#endif /* math.h  */
