/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _PTHREAD_
#define _PTHREAD_

#include <features.h>

#include <sched.h>
#include <time.h>

#define __need_sigset_t
#include <bits/sigset.h>
#include <sys/types.h>

__BEGIN_DECLS

#ifndef _POSIX_THREADS
# define _POSIX_THREADS
#endif
#ifndef _POSIX_THREAD_PRIORITY_SCHEDULING
# define _POSIX_THREAD_PRIORITY_SCHEDULING
#endif

/* Attributes */
#include <bits/pthreaddef.h>

/* Create a thread with given attributes ATTR (or default attributes
   if ATTR is NULL), and call function START_ROUTINE with given
   arguments ARG.  */
EXTERNAPI int pthread_create __P((pthread_t *thread, const pthread_attr_t *attr,
  void *(*start_routine) (void *), void *arg));

/* Obtain the identifier of the current thread.  */
EXTERNAPI pthread_t pthread_self __P((void));

/* Compare two thread identifiers.  */
EXTERNAPI int pthread_equal __P((pthread_t thread1, pthread_t thread2));

/* Terminate calling thread.  */
EXTERNAPI void pthread_exit __P((void *retval)) __attribute__ ((__noreturn__));

/* Make calling thread wait for termination of the thread THREAD.  The
   exit status of the thread is stored in *THREAD_RETURN, if EXIT_VALUE
   is not NULL. */
EXTERNAPI int pthread_join __P((pthread_t thread, void **exit_value));

/* Indicate that the thread THREAD is never to be joined with PTHREAD_JOIN.
   The resources of THREAD will therefore be freed immediately when it
   terminates, instead of waiting for another thread to perform PTHREAD_JOIN
   on it.  */
EXTERNAPI int pthread_detach __P((pthread_t thread));

/* Initialize thread attribute *ATTR with default attributes
   (detachstate is PTHREAD_JOINABLE, scheduling policy is SCHED_RR,
    no user-provided stack).  */
EXTERNAPI int pthread_attr_init __P((pthread_attr_t *attr));

/* Destroy thread attribute *ATTR. */
EXTERNAPI int pthread_attr_destroy __P((pthread_attr_t *attr));

/* Set the `detachstate' attribute in *ATTR according to DETACHSTATE. */
EXTERNAPI int pthread_attr_setdetachstate __P((pthread_attr_t *attr, int detachstate));

/* Return in *DETACHSTATE the `detachstate' attribute in *ATTR. */
EXTERNAPI int pthread_attr_getdetachstate __P((const pthread_attr_t *attr, int *detachstate));

/* Set scheduling parameters (priority, etc) in *ATTR according to PARAM. */
EXTERNAPI int pthread_attr_setschedparam __P((pthread_attr_t *attr,
  const struct sched_param *param));

/* Return in *PARAM the scheduling parameters of *ATTR. */
EXTERNAPI int pthread_attr_getschedparam __P((const pthread_attr_t *attr,
  struct sched_param *param));

/* Set scheduling policy in *ATTR according to POLICY. */
EXTERNAPI int pthread_attr_setschedpolicy __P((pthread_attr_t *attr, int policy));

/* Return in *POLICY the scheduling policy of *ATTR. */
EXTERNAPI int pthread_attr_getschedpolicy __P((const pthread_attr_t *attr,
  int *policy));

/* Set scheduling inheritance mode in *ATTR according to INHERIT.  */
EXTERNAPI int pthread_attr_setinheritsched __P((pthread_attr_t *attr, int inherit));

/* Return in *INHERIT the scheduling inheritance mode of *ATTR.  */
EXTERNAPI int pthread_attr_getinheritsched __P((const pthread_attr_t *attr, int *inherit));

/* Set scheduling contention scope in *ATTR according to SCOPE.  */
EXTERNAPI int pthread_attr_setscope __P((pthread_attr_t *attr, int scope));

/* Return in *SCOPE the scheduling contention scope of *ATTR.  */
EXTERNAPI int pthread_attr_getscope __P((const pthread_attr_t *attr, int *scope));

#ifdef __USE_UNIX98
/* Set the size of the guard area at the bottom of the thread.  */
EXTERNAPI int pthread_attr_setguardsize __P((pthread_attr_t *attr, size_t guardsize));

/* Get the size of the guard area at the bottom of the thread.  */
EXTERNAPI int pthread_attr_getguardsize __P((const pthread_attr_t *attr, size_t *guardsize));
#endif

/* Set the starting address of the stack of the thread to be created.
   Depending on whether the stack grows up or down the value must either
   be higher or lower than all the address in the memory block.  The
   minimal size of the block must be PTHREAD_STACK_SIZE.  */
EXTERNAPI int pthread_attr_setstackaddr __P((pthread_attr_t *attr, void *stackaddr));

/* Return the previously set address for the stack.  */
EXTERNAPI int pthread_attr_getstackaddr __P((const pthread_attr_t *attr, void **stackaddr));

#ifdef __USE_XOPEN2K
/* The following two interfaces are intended to replace the last two.  They
   require setting the address as well as the size since only setting the
   address will make the implementation on some architectures impossible.  */
EXTERNAPI int pthread_attr_setstack __P((pthread_attr_t *attr, void *stackaddr,
  size_t stacksize));

/* Return the previously set address for the stack.  */
EXTERNAPI int pthread_attr_getstack __P((const pthread_attr_t *attr, void **stackaddr,
  size_t *stacksize));
#endif

/* Add information about the minimum stack size needed for the thread
   to be started.  This size must never be less than PTHREAD_STACK_SIZE
   and must also not exceed the system limits.  */
EXTERNAPI int pthread_attr_setstacksize __P((pthread_attr_t *attr, size_t stacksize));

/* Return the currently used minimal stack size.  */
EXTERNAPI int pthread_attr_getstacksize __P((const pthread_attr_t *attr, size_t *stacksize));

#ifdef __USE_GNU
/* Get thread attributes corresponding to the already running thread THREAD.  */
EXTERNAPI int pthread_getattr_np __P((pthread_t thread, pthread_attr_t *attr));
#endif

/* Functions for scheduling control.  */

/* Set the scheduling parameters for THREAD according to POLICY
   and *PARAM.  */
EXTERNAPI int pthread_setschedparam __P((pthread_t thread, int policy,
  const struct sched_param *param));

/* Return in *POLICY and *PARAM the scheduling parameters for THREAD.  */
EXTERNAPI int pthread_getschedparam __P((pthread_t thread, int *policy,
  struct sched_param *param));

#ifdef __USE_UNIX98
/* Determine level of concurrency.  */
EXTERNAPI int pthread_getconcurrency __P((void));

/* Set new concurrency level to LEVEL.  */
EXTERNAPI int pthread_setconcurrency __P((int level));
#endif

#ifdef __USE_GNU
/* Yield the processor to another thread or process.
   This function is similar to the POSIX `sched_yield' function but
   might be differently implemented in the case of a m-on-n thread
   implementation.  */
EXTERNAPI int pthread_yield __P((void));
#endif

/* Functions for mutex handling.  */

/* Initialize MUTEX using attributes in *MUTEX_ATTR, or use the
   default values if later is NULL.  */
EXTERNAPI int pthread_mutex_init __P((pthread_mutex_t *mutex,
  const pthread_mutexattr_t *mutex_attr));

/* Destroy MUTEX.  */
EXTERNAPI int pthread_mutex_destroy __P((pthread_mutex_t *mutex));

/* Try to lock MUTEX.  */
EXTERNAPI int pthread_mutex_trylock __P((pthread_mutex_t *mutex));

/* Wait until lock for MUTEX becomes available and lock it.  */
EXTERNAPI int pthread_mutex_lock __P((pthread_mutex_t *mutex));

#ifdef __USE_XOPEN2K
/* Wait until lock becomes available, or specified time passes. */
EXTERNAPI int pthread_mutex_timedlock __P((pthread_mutex_t *mutex,
  const struct timespec *timeout));
#endif

/* Unlock MUTEX.  */
EXTERNAPI int pthread_mutex_unlock __P((pthread_mutex_t *mutex));

/* Initialize mutex attribute object ATTR with default attributes
   (kind is PTHREAD_DEFAULT).  */
EXTERNAPI int pthread_mutexattr_init __P((pthread_mutexattr_t *attr));

/* Destroy mutex attribute object ATTR.  */
EXTERNAPI int pthread_mutexattr_destroy __P((pthread_mutexattr_t *attr));

/* Get the process-shared flag of the mutex attribute ATTR.  */
EXTERNAPI int pthread_mutexattr_getpshared __P((const pthread_mutexattr_t *attr,
  int *pshared));

/* Set the process-shared flag of the mutex attribute ATTR.  */
EXTERNAPI int pthread_mutexattr_setpshared __P((pthread_mutexattr_t *attr,
  int pshared));

#ifdef __USE_UNIX98
/* Set the mutex kind attribute in *ATTR to KIND (either PTHREAD_MUTEX_NORMAL,
   PTHREAD_MUTEX_RECURSIVE, PTHREAD_MUTEX_ERRORCHECK, or
   PTHREAD_MUTEX_DEFAULT).  */
EXTERNAPI int pthread_mutexattr_settype __P((pthread_mutexattr_t *attr, int kind));

/* Return in *KIND the mutex kind attribute in *ATTR.  */
EXTERNAPI int pthread_mutexattr_gettype __P((const pthread_mutexattr_t *attr,
  int *kind));
#endif

/* Get autoinitialize in the child process after fork.  */
EXTERNAPI int pthread_mutexattr_getkind_np __P((const pthread_mutexattr_t *attr,
  int *pref));

/* Set or clear it.  */
EXTERNAPI int pthread_mutexattr_setkind_np __P((pthread_mutexattr_t *attr, int pref));

/* Initialize condition variable COND using attributes ATTR, or use
   the default values if later is NULL.  */
EXTERNAPI int pthread_cond_init __P((pthread_cond_t *cond, const pthread_condattr_t *attr));

/* Destroy condition variable COND.  */
EXTERNAPI int pthread_cond_destroy __P((pthread_cond_t *cond));

/* Wake up one thread waiting for condition variable COND.  */
EXTERNAPI int pthread_cond_signal __P((pthread_cond_t *cond));

/* Wake up all threads waiting for condition variables COND.  */
EXTERNAPI int pthread_cond_broadcast __P((pthread_cond_t *cond));

/* Wait for condition variable COND to be signaled or broadcast.
   MUTEX is assumed to be locked before.  */
EXTERNAPI int pthread_cond_wait __P((pthread_cond_t *cond, pthread_mutex_t *mutex));

/* Wait for condition variable COND to be signaled or broadcast until
   ABSTIME.  MUTEX is assumed to be locked before.  ABSTIME is an
   absolute time specification; zero is the beginning of the epoch
   (00:00:00 GMT, January 1, 1970).  */
EXTERNAPI int pthread_cond_timedwait __P((pthread_cond_t *cond, pthread_mutex_t *mutex,
  const struct timespec *timeout));

/* Initialize condition variable attribute ATTR.  */
EXTERNAPI int pthread_condattr_init __P((pthread_condattr_t *attr));

/* Destroy condition variable attribute ATTR.  */
EXTERNAPI int pthread_condattr_destroy __P((pthread_condattr_t *attr));

/* Get the process-shared flag of the condition variable attribute ATTR.  */
EXTERNAPI int pthread_condattr_getpshared __P((const pthread_condattr_t *attr, int *pshared));

/* Set the process-shared flag of the condition variable attribute ATTR.  */
EXTERNAPI int pthread_condattr_setpshared __P((pthread_condattr_t *attr, int pshared));

#ifdef __USE_UNIX98
/* Initialize read-write lock RWLOCK using attributes ATTR, or use
   the default values if later is NULL.  */
EXTERNAPI int pthread_rwlock_init __P((pthread_rwlock_t *rwlock,
  const pthread_rwlockattr_t *attr));

/* Destroy read-write lock RWLOCK.  */
EXTERNAPI int pthread_rwlock_destroy __P((pthread_rwlock_t *rwlock));

/* Acquire read lock for RWLOCK.  */
EXTERNAPI int pthread_rwlock_rdlock __P((pthread_rwlock_t *rwlock));

/* Try to acquire read lock for RWLOCK.  */
EXTERNAPI int pthread_rwlock_tryrdlock __P((pthread_rwlock_t *rwlock));

#ifdef __USE_XOPEN2K
/* Try to acquire read lock for RWLOCK or return after specfied time.  */
EXTERNAPI int pthread_rwlock_timedrdlock __P((pthread_rwlock_t *rwlock,
  const struct timespec *timeout));
#endif

/* Acquire write lock for RWLOCK.  */
EXTERNAPI int pthread_rwlock_wrlock __P((pthread_rwlock_t *rwlock));

/* Try to acquire write lock for RWLOCK.  */
EXTERNAPI int pthread_rwlock_trywrlock __P((pthread_rwlock_t *rwlock));

#ifdef __USE_XOPEN2K
/* Try to acquire write lock for RWLOCK or return after specfied time.  */
EXTERNAPI int pthread_rwlock_timedwrlock __P((pthread_rwlock_t *rwlock,
  const struct timespec *timeout));
#endif

/* Unlock RWLOCK.  */
EXTERNAPI int pthread_rwlock_unlock __P((pthread_rwlock_t *rwlock));


/* Functions for handling read-write lock attributes.  */

/* Initialize attribute object ATTR with default values.  */
EXTERNAPI int pthread_rwlockattr_init __P((pthread_rwlockattr_t *attr));

/* Destroy attribute object ATTR.  */
EXTERNAPI int pthread_rwlockattr_destroy __P((pthread_rwlockattr_t *attr));

/* Return current setting of process-shared attribute of ATTR in PSHARED.  */
EXTERNAPI int pthread_rwlockattr_getpshared __P((const pthread_rwlockattr_t *attr,
  int *pshared));

/* Set process-shared attribute of ATTR to PSHARED.  */
EXTERNAPI int pthread_rwlockattr_setpshared __P((pthread_rwlockattr_t *attr,
  int pshared));

/* Get autoinitialize in the child process after fork.  */
EXTERNAPI int pthread_rwlockattr_getkind_np __P((const pthread_rwlockattr_t *attr,
  int *pref));

/* Set or clear it.  */
EXTERNAPI int pthread_rwlockattr_setkind_np __P((pthread_rwlockattr_t *attr, int pref));
#endif

#ifdef __USE_XOPEN2K
/* The IEEE Std. 1003.1j-2000 introduces functions to implement
   spinlocks.  */

/* Initialize the spinlock LOCK.  If PSHARED is nonzero the spinlock can
   be shared between different processes.  */
EXTERNAPI int pthread_spin_init __P((pthread_spinlock_t *lock, int pshared));

/* Destroy the spinlock LOCK.  */
EXTERNAPI int pthread_spin_destroy __P((pthread_spinlock_t *lock));

/* Wait until spinlock LOCK is retrieved.  */
EXTERNAPI int pthread_spin_lock __P((pthread_spinlock_t *lock));
EXTERNAPI int pthread_spin_timedlock_np __P((pthread_spinlock_t *lock,
  const struct timespec *timeout));

/* Try to lock spinlock LOCK.  */
EXTERNAPI int pthread_spin_trylock __P((pthread_spinlock_t *lock));

/* Release spinlock LOCK.  */
EXTERNAPI int pthread_spin_unlock __P((pthread_spinlock_t *__lock));


/* Barriers are a also a new feature in 1003.1j-2000. */

EXTERNAPI int pthread_barrier_init __P((pthread_barrier_t *barrier,
  const pthread_barrierattr_t *attr, unsigned int count));

EXTERNAPI int pthread_barrier_destroy __P((pthread_barrier_t *barrier));

EXTERNAPI int pthread_barrierattr_init __P((pthread_barrierattr_t *attr));

EXTERNAPI int pthread_barrierattr_destroy __P((pthread_barrierattr_t *attr));

EXTERNAPI int pthread_barrierattr_getpshared __P((const pthread_barrierattr_t *attr,
  int *pshared));

EXTERNAPI int pthread_barrierattr_setpshared __P((pthread_barrierattr_t *attr,
  int pshared));

EXTERNAPI int pthread_barrier_wait __P((pthread_barrier_t *barrier));
#endif


/* Create a key value identifying a location in the thread-specific
   data area.  Each thread maintains a distinct thread-specific data
   area.  DESTR_FUNCTION, if non-NULL, is called with the value
   associated to that key when the key is destroyed.
   DESTR_FUNCTION is not called if the value associated is NULL when
   the key is destroyed.  */
EXTERNAPI int pthread_key_create __P((pthread_key_t *key, void (*destr_function) (void *)));

/* Destroy KEY.  */
EXTERNAPI int pthread_key_delete __P((pthread_key_t key));

/* Store POINTER in the thread-specific data slot identified by KEY. */
EXTERNAPI int pthread_setspecific __P((pthread_key_t key, const void *pointer));

/* Return current value of the thread-specific data slot identified by KEY.  */
EXTERNAPI void *pthread_getspecific __P((pthread_key_t key));


/* Install a cleanup handler: ROUTINE will be called with arguments ARG
   when the thread is cancelled or calls pthread_exit.  ROUTINE will also
   be called with arguments ARG when the matching pthread_cleanup_pop
   is executed with non-zero EXECUTE argument.
   pthread_cleanup_push and pthread_cleanup_pop are macros and must always
   be used in matching pairs at the same nesting level of braces.
   Based on SEH so you can use it with SEH and SEH based C++ exception
   handling (MSVC like for example).
 */

#define pthread_cleanup_push(_A_routine, _A_userarg) \
  { NTTH_CLEANUP_BUFFER __L_buffer; \
    __rt_push_cleanup_buffer (&__L_buffer, _A_routine, _A_userarg, 0)

EXTERNAPI void __rt_push_cleanup_buffer __P((PNTTH_CLEANUP_BUFFER buffer,
  void (*routine) (void*), void *userarg, int defer));

/* Remove a cleanup handler installed by the matching pthread_cleanup_push.
   If EXECUTE is non-zero, the handler function is called. */

#define pthread_cleanup_pop(ExecuteArg) \
    __rt_pop_cleanup_buffer (ExecuteArg, 0); }

EXTERNAPI void __rt_pop_cleanup_buffer __P((int execute, int defer));

/* Install a cleanup handler as pthread_cleanup_push does, but also
   saves the current cancellation type and set it to deferred cancellation.  */

#ifdef __USE_GNU
# define pthread_cleanup_push_defer_np(_A_routine, _A_userarg) \
  { NTTH_CLEANUP_BUFFER __L_buffer; \
    __rt_push_cleanup_buffer (&__L_buffer, _A_routine, _A_userarg, 1)

/* Remove a cleanup handler as pthread_cleanup_pop does, but also
   restores the cancellation type that was in effect when the matching
   pthread_cleanup_push_defer was called.  */

# define pthread_cleanup_pop_restore_np(ExecuteArg) \
  __rt_pop_cleanup_buffer (ExecuteArg, 1); }
#endif


/* Guarantee that the initialization function INIT_ROUTINE will be called
   only once, even if pthread_once is executed several times with the
   same ONCE_CONTROL argument. ONCE_CONTROL must point to a static or
   EXTERNAPI variable initialized to PTHREAD_ONCE_INIT.  */
EXTERNAPI int pthread_once __P((pthread_once_t *once_control, void (*init_routine) (void)));


/* Set cancelability state of current thread to STATE, returning old
   state in *OLDSTATE if OLDSTATE is not NULL.  */
EXTERNAPI int pthread_setcancelstate __P((int state, int *oldstate));

/* Set cancellation state of current thread to TYPE, returning the old
   type in *OLDTYPE if OLDTYPE is not NULL.  */
EXTERNAPI int pthread_setcanceltype __P((int type, int *oldtype));

/* Cancel THREAD immediately or at the next possibility.  */
EXTERNAPI int pthread_cancel __P((pthread_t thread));

/* Test for pending cancellation for the current thread and terminate
   the thread as per pthread_exit(PTHREAD_CANCELED) if it has been
   cancelled.  */
EXTERNAPI void pthread_testcancel __P((void));

#ifdef __USE_XOPEN2K
/* Get ID of CPU-time clock for thread THREAD_ID.  */
EXTERNAPI int pthread_getcpuclockid __P((pthread_t thread_id, clockid_t *clock_id));
#endif

#include <bits/sigset.h>

EXTERNAPI int pthread_sigmask __P((int how, const sigset_t *newmask, sigset_t *oldmask));

/* Send signal SIGNO to the given thread. */
EXTERNAPI int pthread_kill __P((pthread_t thread, int signo));

/* Install handlers to be called when a new process is created with FORK.
   The PREPARE handler is called in the parent process just before performing
   FORK. The PARENT handler is called in the parent process just after FORK.
   The CHILD handler is called in the child process.  Each of the three
   handlers can be NULL, meaning that no handler needs to be called at that
   point.
   PTHREAD_ATFORK can be called several times, in which case the PREPARE
   handlers are called in LIFO order (last added with PTHREAD_ATFORK,
   first called before FORK), and the PARENT and CHILD handlers are called
   in FIFO (first added, first called).  */

EXTERNAPI int pthread_atfork __P((void (*prepare) (void), void (*parent) (void),
  void (*child) (void)));

/* Terminate all threads in the program except the calling process.
   Should be called just before invoking one of the exec*() functions.  */
EXTERNAPI void pthread_kill_other_threads_np __P((void));

/* Allocate and return topdown floating stack. RESERVE and COMMIT may be zero in
   this case using default values RESERVE = process defult stack size,
   commit = 2 * page + guard page. Return 0 at success or error code else. */
EXTERNAPI int pthread_allocate_stack_np __P((size_t reserve, size_t commit, void **st_base));

EXTERNAPI int pthread_set_infinite_timeout_np __P((struct timespec *timeout));

__END_DECLS

#endif	/* pthread.h */
