/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _SEMAPHORE_
#define _SEMAPHORE_

#include <features.h>
#include <sys/types.h>
#ifdef __USE_XOPEN2K
# define __need_timespec
# include <time.h>
#endif

typedef struct _SEMAPHORE sem_t;

/* Value returned if `sem_open' failed.  */
#define SEM_FAILED ((sem_t *) 0)

#ifndef _LIBC_LIMITS_
/* Maximum value the semaphore can have.  */
# define SEM_VALUE_MAX ((int) ((~0u) >> 1))
#endif

__BEGIN_DECLS

/* Initialize semaphore object SEM to VALUE.  If PSHARED then share it
   with other processes.  */
EXTERNAPI int sem_init __P((sem_t *sem, int pshared, unsigned int value));

/* Free resources associated with semaphore object SEM.  */
EXTERNAPI int sem_destroy __P((sem_t *sem));

/* Open a named semaphore NAME with open flaot OFLAG.  */
EXTERNAPI sem_t *sem_open __P((const char *name, int oflag, ...));

/* Close descriptor for named semaphore SEM. */
EXTERNAPI int sem_close __P((sem_t *sem));

/* Remove named semaphore NAME.  */
EXTERNAPI int sem_unlink __P((const char *name));

/* Wait for SEM being posted.  */
EXTERNAPI int sem_wait __P((sem_t *sem));

#ifdef __USE_XOPEN2K
/* Similar to `sem_wait' but wait only until ABSTIME.  */
EXTERNAPI int sem_timedwait __P((sem_t *sem, const struct timespec *abstime));
#endif

/* Test whether SEM is posted.  */
EXTERNAPI int sem_trywait __P((sem_t *sem));

/* Post SEM.  */
EXTERNAPI int sem_post __P((sem_t *sem));

/* Get current value of SEM and store it in *SVAL.  */
EXTERNAPI int sem_getvalue __P((sem_t *sem, int *sval));

__END_DECLS

#endif
