/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _SIGNAL_
#define _SIGNAL_

#include <features.h>

#ifdef _WIN64
# pragma pack (push, 8)
#else
# pragma pack (push, 4)
#endif

#ifdef __BORLANDC__
# pragma anon_struct on
#endif

__BEGIN_DECLS

#include <bits/sigset.h>

/* An integral type that can be modified atomically, without the
   possibility of a signal arriving in the middle of the operation.  */
#ifndef __sig_atomic_t_defined
typedef unsigned long sig_atomic_t;
# define __sig_atomic_t_defined
#endif

#ifdef __need_sig_atomic_t
# undef __need_sig_atomic_t
#endif

#include <bits/signum.h>

#ifndef __pid_t_defined
typedef long int pid_t;
# define __pid_t_defined
#endif

#ifndef __uid_t_defined
typedef unsigned long uid_t;
# define __uid_t_defined
#endif

/* The X/Open definition of `signal' specifies the SVID semantic.  Use
   the additional function `sysv_signal' when X/Open compatibility is
   requested.  */
#ifdef __USE_GNU
EXTERNAPI void (*sysv_signal __P((int sig, void (*handler) (int)))) (int);
#endif

/* Set the handler for the signal SIG to HANDLER, returning the old
   handler, or SIG_ERR on error.
   By default `signal' has the BSD semantic.  */
#if defined __USE_BSD || defined _BSD_SIGNALS || defined _BSD_COMPAT
# if defined _BSD_SIGNALS || defined _BSD_COMPAT
EXTERNAPI int (*signal __P((int sig, int (*func)(int, ...)))) (int, ...);
# else
EXTERNAPI void (*signal __P((int sig, void (*handler) (int)))) (int);
# endif
#else
# define signal sysv_signal
#endif

#ifdef __USE_XOPEN
/* The X/Open definition of `signal' conflicts with the BSD version.
   So they defined another function `bsd_signal'.  */
EXTERNAPI void (*bsd_signal __P((int sig, void (*handler) (int)))) (int);
#endif

/* Send signal SIG to process number PID.  If PID is zero,
   send SIG to all processes in the current process's process group.
   If PID is < -1, send SIG to all processes in process group - PID.  */
#ifdef __USE_POSIX
EXTERNAPI int kill __P((pid_t pid, int sig));
#endif /* Use POSIX.  */

#if defined __USE_BSD || defined __USE_XOPEN_EXTENDED
/* Send SIG to all processes in process group PGRP.
   If PGRP is zero, send SIG to all processes in
   the current process's process group.  */
EXTERNAPI int killpg __P((pid_t pgrp, int sig));
#endif /* Use BSD || X/Open Unix.  */

/* Raise signal SIG, i.e., send SIG to yourself.  */
EXTERNAPI int raise __P((int sig));

#ifdef __USE_SVID
/* SVID names for the same things.  */
EXTERNAPI void (*ssignal __P((int sig, void (*handler) (int)))) (int);
EXTERNAPI int gsignal __P((int sig));
#endif /* Use SVID.  */

#ifdef __USE_MISC
/* Print a message describing the meaning of the given signal number.  */
EXTERNAPI void psignal __P((int sig, const char *s));
#endif /* Use misc.  */


/* The `sigpause' function has two different interfaces.  The original
   BSD definition defines the argument as a mask of the signal, while
   the more modern interface in X/Open defines it as the signal
   number.  We go with the BSD version unless the user explicitly
   selects the X/Open version.  */
#ifdef __USE_BSD
/* Set the mask of blocked signals to MASK,
   wait for a signal to arrive, and then restore the mask.  */
EXTERNAPI int _sigpause __P((int mask));
# define sigpause(__sig_arg) _sigpause (__sig_arg)
#elif defined __USE_XOPEN
/* Remove a signal from the signal mask and suspend the process.  */
EXTERNAPI int sigpause __P((int sig));
#endif


#ifdef __USE_BSD
/* None of the following functions should be used anymore.  They are here
   only for compatibility.  A single word (`int') is not guaranteed to be
   enough to hold a complete signal mask and therefore these functions
   simply do not work in many situations.  Use `sigprocmask' instead.  */

/* Block signals in MASK, returning the old mask.  */
EXTERNAPI int sigblock __P((int mask));

/* Set the mask of blocked signals to MASK, returning the old mask.  */
EXTERNAPI int sigsetmask __P((int mask));

/* Return currently selected signal mask.  */
EXTERNAPI int siggetmask __P((void));
#endif /* Use BSD.  */


#ifdef __USE_MISC
# define NSIG	_NSIG
#endif

#ifdef __USE_GNU
typedef void (*sighandler_t) (int);
#endif

/* 4.4 BSD uses the name `sig_t' for this.  */
#ifdef __USE_BSD
typedef void (*sig_t) (int);
#endif

#ifdef __USE_POSIX

# ifdef __USE_POSIX199309
/* We need `struct timespec' later on.  */
#  define __need_timespec
#  define __need_clock_t
#  include <time.h>

#include <bits/siginfo.h>
#include <bits/sigevent.h>

# endif

__inline int sigemptyset (sigset_t *set)
{
  int i;

  for (i = 0; i < _SIGSET_NWORDS; i += 4) {
    set->__val[i + 0] = 0;
    set->__val[i + 1] = 0;
    set->__val[i + 2] = 0;
    set->__val[i + 3] = 0;
  }

  return 0;
}

__inline int sigfillset (sigset_t *set)
{
  int i;

  for (i = 0; i < _SIGSET_NWORDS; i += 4) {
    set->__val[i + 0] = ~0U;
    set->__val[i + 1] = ~0U;
    set->__val[i + 2] = ~0U;
    set->__val[i + 3] = ~0U;
  }

  return 0;
}

EXTERNAPI int sigismember (const sigset_t *set, int sig);

EXTERNAPI int sigaddset (sigset_t *set, int sig);

EXTERNAPI int sigdelset (sigset_t *set, int sig);

# ifdef __USE_GNU
/* The POSIX does not specify for handling the whole signal set in one
   command.  This is often wanted and so we define three more functions
   here.  */

/* Return non-empty value is SET is not empty.  */
__inline int sigisemptyset (const sigset_t *set) 
{
  int i;
  unsigned long int accum;

  for (i = 0, accum = 0; !accum && i < _SIGSET_NWORDS; i += 4) {
    accum |= set->__val[i + 0];
    accum |= set->__val[i + 1];
    accum |= set->__val[i + 2];
    accum |= set->__val[i + 3];
  }

  return accum == 0;
}

/* Build new signal set by combining the two inputs set using logical AND.  */
__inline int
sigandset (
    sigset_t *set,
    const sigset_t *left,
    const sigset_t *right
  )
{
  int i;

  for (i = 0; i < _SIGSET_NWORDS; i += 4) {
    set->__val[i + 0] = left->__val[i + 0] & right->__val[i + 0];
    set->__val[i + 1] = left->__val[i + 1] & right->__val[i + 1];
    set->__val[i + 2] = left->__val[i + 2] & right->__val[i + 2];
    set->__val[i + 3] = left->__val[i + 3] & right->__val[i + 3];
  }

  return 0;
}

/* Build new signal set by combining the two inputs set using logical OR.  */
__inline int
sigorset (
    sigset_t *set,
    const sigset_t *left,
    const sigset_t *right
  )
{
  int i;

  for (i = 0; i < _SIGSET_NWORDS; i += 4) {
    set->__val[i + 0] = left->__val[i + 0] | right->__val[i + 0];
    set->__val[i + 1] = left->__val[i + 1] | right->__val[i + 1];
    set->__val[i + 2] = left->__val[i + 2] | right->__val[i + 2];
    set->__val[i + 3] = left->__val[i + 3] | right->__val[i + 3];
  }

  return 0;
}

# endif /* GNU */

#include <bits/sigaction.h>

/* Get and/or change the set of blocked signals.  */
EXTERNAPI int sigprocmask __P((int how, const sigset_t *set, sigset_t *oset));

/* Change the set of blocked signals to SET,
   wait until a signal arrives, and restore the set of blocked signals.  */
EXTERNAPI int sigsuspend __P((const sigset_t *set));

/* Get and/or set the action for signal SIG.  */
EXTERNAPI int sigaction __P((int sig, const struct sigaction *act, struct sigaction *oact));

/* Put in SET all signals that are blocked and waiting to be delivered.  */
EXTERNAPI int sigpending __P((sigset_t *set));

/* Select any of pending signals from SET or wait for any to arrive.  */
EXTERNAPI int sigwait __P((const sigset_t *set, int *sig));

# ifdef __USE_POSIX199309
/* Select any of pending signals from SET and place information in INFO.  */
EXTERNAPI int sigwaitinfo __P((const sigset_t *set, siginfo_t *info));

/* Select any of pending signals from SET and place information in INFO.
   Wait the imte specified by TIMEOUT if no signal is pending.  */
EXTERNAPI int sigtimedwait __P((const sigset_t *set, siginfo_t *info, const struct timespec *tm));

/* Send signal SIG to the process PID.  Associate data in VAL with the
   signal.  */
EXTERNAPI int sigqueue __P((pid_t pid, int sig, const union sigval val));
# endif	/* Use POSIX 199306.  */

#endif /* Use POSIX.  */

#ifdef __USE_BSD

/* Names of the signals.  This variable exists only for compatibility.
   Use `strsignal' instead (see <string.h>).  */
#define _sys_siglist sys_siglist
EXTERNAPI extern const char *const sys_siglist[_NSIG];

#include <bits/sigvec.h>

/* If VEC is non-NULL, set the handler for SIG to the `sv_handler' member
   of VEC.  The signals in `sv_mask' will be blocked while the handler runs.
   If the SV_RESETHAND bit is set in `sv_flags', the handler for SIG will be
   reset to SIG_DFL before `sv_handler' is entered.  If OVEC is non-NULL,
   it is filled in with the old information for SIG.  */
EXTERNAPI int sigvec __P((int sig, const struct sigvec *vec, struct sigvec *ovec));

/* Restore the state saved in SCP.  */
# ifndef _UCONTEXT_
#  define sigcontext ucontext
struct ucontext;
# endif
EXTERNAPI int sigreturn __P((struct sigcontext *scp));

#endif /*  use BSD.  */


#if defined __USE_BSD || defined __USE_XOPEN_EXTENDED

/* If INTERRUPT is nonzero, make signal SIG interrupt system calls
   (causing them to fail with EINTR); if INTERRUPT is zero, make system
   calls be restarted after signal SIG.  */
EXTERNAPI int siginterrupt __P((int sig, int interrupt));

/* Structure describing a signal stack (obsolete). */
struct sigstack {
  void *ss_sp; /* Signal stack pointer. */
  int ss_onstack; /* Nonzero if executing on this stack. */
};

#include <bits/sigaltstack.h>
# ifdef __USE_XOPEN
#  include <ucontext.h>
# endif

/* Run signals handlers on the stack specified by SS (if not NULL).
   If OSS is not NULL, it is filled in with the old signal stack status.
   This interface is obsolete and on many platform not implemented. */
EXTERNAPI int sigstack __P((const struct sigstack *ss, struct sigstack *oss));

/* Alternate signal handler stack interface.
   This interface should always be preferred over `sigstack'. */
EXTERNAPI int sigaltstack __P((const stack_t *ss, stack_t *oss));

#endif /* use BSD or X/Open Unix. */

#ifdef __USE_UNIX98
/* Simplified interface for signal management. */

/* Add SIG to the calling process' signal mask. */
EXTERNAPI int sighold __P((int sig));

/* Remove SIG from the calling process' signal mask. */
EXTERNAPI int sigrelse __P((int sig));

/* Set the disposition of SIG to SIG_IGN.  */
EXTERNAPI int sigignore __P((int sig));

/* Set the disposition of SIG.  */
EXTERNAPI void (*sigset __P((int sig, void (*disp) (int)))) (int);
#endif

#if defined __USE_POSIX199506 || defined __USE_UNIX98
/* Some of the functions for handling signals in threaded programs must
   be defined here.  */
/* Modify the signal mask for the calling thread.  The arguments have the
   same meaning as for sigprocmask; in fact, this and sigprocmask might be
   the same function.  We declare this the same on all platforms, since it
   doesn't use any thread-related types.  */
EXTERNAPI int pthread_sigmask __P((int how, const sigset_t *newmask, sigset_t *oldmask));
#endif /* use Unix98 */


__END_DECLS

#pragma pack (pop)

#endif
