/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _SYS_TIME_
#define _SYS_TIME_

#include <features.h>

#define __need_time_t
#include <time.h>
#include <bits/timeval.h>
#include <sys/select.h>


__BEGIN_DECLS

#ifdef __USE_GNU
/* Macros for converting between `struct timeval' and `struct timespec'.  */
# define TIMEVAL_TO_TIMESPEC(tv, ts) {                                  \
        (ts)->tv_sec = (tv)->tv_sec;                                    \
        (ts)->tv_nsec = (tv)->tv_usec * 1000;                           \
}
# define TIMESPEC_TO_TIMEVAL(tv, ts) {                                  \
        (tv)->tv_sec = (ts)->tv_sec;                                    \
        (tv)->tv_usec = (ts)->tv_nsec / 1000;                           \
}
#endif


#ifdef __USE_BSD
/* Structure crudely representing a timezone.
   This is obsolete and should never be used.  */
struct timezone {
  int tz_minuteswest;  /* Minutes west of GMT.  */
  int tz_dsttime;      /* Nonzero if DST is ever in effect.  */
};

typedef struct timezone * __timezone_ptr_t;
#else
typedef void * __timezone_ptr_t;
#endif

/* Get the current time of day and timezone information,
   putting it into *TV and *TZ.  If TZ is NULL, *TZ is not filled.
   Returns 0 on success, -1 on errors.
   NOTE: This form of timezone information is obsolete.
   Use the functions and variables declared in <time.h> instead.  */
EXTERNAPI int gettimeofday __P((struct timeval *tv, __timezone_ptr_t tz));

#ifdef __USE_BSD
/* Set the current time of day and timezone information.
   This call is restricted to the super-user.  */
EXTERNAPI int settimeofday __P((const struct timeval *tv, const struct timezone *tz));

/* Adjust the current time of day by the amount in DELTA.
   If OLDDELTA is not NULL, it is filled in with the amount
   of time adjustment remaining to be done from the last `adjtime' call.
   This call is restricted to the super-user.  */
EXTERNAPI int adjtime __P((const struct timeval *delta, struct timeval *olddelta));
#endif


/* Values for the first argument to `getitimer' and `setitimer'.  */
 /* Timers run in real time.  */
#define ITIMER_REAL 0
/* Timers run only when the process is executing.  */
#define ITIMER_VIRTUAL 1
/* Timers run when the process is executing and when
   the system is executing on behalf of the process.  */
#define ITIMER_PROF 2

/* Type of the second argument to `getitimer' and
   the second and third arguments `setitimer'.  */
struct itimerval {
  /* Value to put into `it_value' when the timer expires.  */
  struct timeval it_interval;
  /* Time to the next timer expiration.  */
  struct timeval it_value;
};

/* Set *VALUE to the current setting of timer WHICH.
   Return 0 on success, -1 on errors.  */
EXTERNAPI int getitimer __P((int which, struct itimerval *value));

/* Set the timer WHICH to *NEW.  If OLD is not NULL,
   set *OLD to the old value of timer WHICH.
   Returns 0 on success, -1 on errors.  */
EXTERNAPI int setitimer __P((int which, const struct itimerval *newval, struct itimerval *oldval));

/* Change the access time of FILE to TVP[0] and
   the modification time of FILE to TVP[1].  */
EXTERNAPI int utimes __P((const char *file, const struct timeval tvp[2]));

#ifdef __USE_BSD
/* Convenience macros for operations on timevals.
   NOTE: `timercmp' does not work for >= or <=.  */
# define timerisset(tvp)	((tvp)->tv_sec || (tvp)->tv_usec)
# define timerclear(tvp)	((tvp)->tv_sec = (tvp)->tv_usec = 0)
# define timercmp(a, b, CMP) \
  (((a)->tv_sec == (b)->tv_sec) ? \
   ((a)->tv_usec CMP (b)->tv_usec) : \
   ((a)->tv_sec CMP (b)->tv_sec))
# define timeradd(a, b, result) \
  do { \
    (result)->tv_sec = (a)->tv_sec + (b)->tv_sec; \
    (result)->tv_usec = (a)->tv_usec + (b)->tv_usec; \
    if ((result)->tv_usec >= 1000000)	{ \
      ++(result)->tv_sec; \
      (result)->tv_usec -= 1000000; \
    } \
  } while (0)
# define timersub(a, b, result) \
  do { \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec; \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec; \
    if ((result)->tv_usec < 0) { \
      --(result)->tv_sec; \
      (result)->tv_usec += 1000000; \
    } \
  } while (0)
#endif	/* BSD */

__END_DECLS

#endif /* sys/time.h */
