/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _SYSUCONTEXT_
#define _SYSUCONTEXT_

#pragma pack (push, 4)
#include <sys/types.h>
#include <bits/sigset.h>
#include <bits/sigaltstack.h>

#ifndef __mcontext_defined

# if defined _X86_ || defined __i386__ || defined _M_IX86

#  pragma pack (push, 16)
/* Container for all general registers.  */
typedef struct gregset {
  int seg_gs;
  int seg_es;
  int seg_ss;
  int seg_ds;
  int seg_fs;
  int seg_cs;

  int eax;
  int ebx;
  int ecx;
  int edx;
  int edi;
  int esi;
  int ebp;
  int esp;

  int eflg;
  int eip;
} gregset_t;

/* Structure to describe FPU registers.  */
typedef struct fpregset {
  unsigned long int cw;
  unsigned long int sw;
  unsigned long int tag;
  unsigned long int ipoff;
  unsigned long int cssel;
  unsigned long int dataoff;
  unsigned long int datasel;
  struct _st {
    unsigned short int significand[4];
    unsigned short int exponent;
  } st[8];
} fpregset_t;

/* Structure to describe FP, MMX or XMM registers.  */
typedef struct xfpregset {
  unsigned short int fcw;
  unsigned short int fsw;
  unsigned short int ftw;
  unsigned short int fop;
  unsigned long int ip;
  unsigned long int cs;
  unsigned long int dp;
  unsigned long int ds;
  unsigned long int mxcsr;
  unsigned long int reserved;
  union _st_mm {
    struct _xst {
      unsigned short int significand[4];
      unsigned short int exponent;
      unsigned short int pad[3];
    } xst;
    struct _mm8 {
      unsigned char mm[8];
      unsigned long int pad[2];
    } mm8;
    struct _mm16 {
      unsigned short int mm[4];
      unsigned long int pad[2];
    } mm16;
    struct _mm32 {
      unsigned long int mm[2];
      unsigned long int pad[2];
    } mm32;
    struct _mm64 {
      unsigned __int64 mm;
      unsigned long int pad[2];
    } mm64;
  } st_mm[8]; /*128 bytes */
  union _xmm {
    float xflt[4];
    double xdbl[2];
    unsigned char xbt[16];
    unsigned short int xwd[8];
    unsigned long int xdw[4];
    unsigned __int64 xqw[2];
  } xmm[8];
  long int padding[56];
} xfpregset_t;

/* Context to describe whole processor state.  */
#define __MCONTEXT_NOFPREGSET 0x0
#define __MCONTEXT_FPREGSET   0x1
#define __MCONTEXT_XFPREGSET  0x2 /* with SSE (PIII, PIV) */

typedef struct mcontext {
  int fp_flags;
  union _fpregs {
    fpregset_t regs;
    xfpregset_t xregs;
  } fpregs;
  gregset_t gregs;
} mcontext_t;

#  pragma pack (pop)
# else /* _X86_ */
#  error Depended code.
# endif

# define __mcontext_defined
#endif

#ifndef __ucontext_defined
/* Userlevel context.  */
typedef struct ucontext {
  struct ucontext *uc_link;
  sigset_t uc_sigmask;
  stack_t uc_stack;
  mcontext_t *uc_mcontext; /* non zero only when thread was in the user mode */
} ucontext_t;

#ifndef __sigcontext_defined
typedef struct ucontext sigcontext;
# define __sigcontext_defined
#endif

# define __ucontext_defined
#endif

#pragma pack (pop)

#endif /* sys/ucontext.h */