/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 *	ISO C99 Standard: 7.23 Date and time	<time.h>
 */

#include <features.h>

#ifndef	_TIME_
#if !(defined __need_time_t || defined __need_clock_t || defined __need_timespec)
# define _TIME_
#endif

#if !defined __clock_t_defined && (defined _TIME_ || defined __need_clock_t)
# define __clock_t_defined
/* Returned by `clock'.  */
typedef __int64 clock_t;
#endif /* clock_t not defined and <time.h> or need clock_t.  */
#undef __need_clock_t

#if !defined __time_t_defined && (defined _TIME_ || defined __need_time_t)
# define __time_t_defined
/* Returned by `time'.  */
typedef long int time_t;
#endif /* time_t not defined and <time.h> or need time_t.  */
#undef __need_time_t

#if !defined __clockid_t_defined && \
   ((defined _TIME_ && defined __USE_POSIX199309) || defined __need_clockid_t)
# define __clockid_t_defined
/* Clock ID used in clock and timer functions.  */
typedef unsigned int clockid_t;
#endif /* clockid_t not defined and <time.h> or need clockid_t.  */
#undef __clockid_time_t

#if !defined __timer_t_defined && \
    ((defined _TIME_ && defined __USE_POSIX199309) || defined __need_timer_t)
# define __timer_t_defined
/* Timer ID returned by `timer_create'.  */
typedef unsigned long int timer_t;
#endif /* timer_t not defined and <time.h> or need timer_t.  */
#undef __need_timer_t


#if !defined __timespec_defined && \
    ((defined _TIME_ && defined __USE_POSIX199309) || defined __need_timespec)
# define __timespec_defined
/* POSIX.1b structure for a time value.  This is like a `struct timeval' but
   has nanoseconds instead of microseconds.  */
struct timespec {
  long int tv_sec;    /* Seconds.  */
  long int tv_nsec;   /* Nanoseconds.  */
};
#endif /* timespec not defined and <time.h> or need timespec.  */
#undef __need_timespec

#ifdef _TIME_

__BEGIN_DECLS

/* Get size_t and NULL from <stddef.h>.  */
# define __need_size_t
# define __need_NULL
# include <stddef.h>

/* ISO/IEC 9899:1990 7.12.1: <time.h>
   The macro `CLOCKS_PER_SEC' is the number per second of the value
   returned by the `clock' function. */
/* CAE XSH, Issue 4, Version 2: <time.h>
   The value of CLOCKS_PER_SEC is required to be 1 million on all
   XSI-conformant systems. */
# define CLOCKS_PER_SEC  1000000L

/* This is the obsolete POSIX.1-1988 name for the same constant.  */
# if !defined __STRICT_ANSI__ && !defined __USE_XOPEN2K
/* NT used 100ns intervals for any timing */
#  define CLK_TCK	10000000L
# endif

# ifdef __USE_POSIX199309
/* Identifier for system-wide realtime clock.  */
#  define CLOCK_REALTIME           0
/* High-resolution timer from the CPU.  */
#  define CLOCK_PROCESS_CPUTIME_ID 2
/* Thread-specific CPU-time clock.  */
#  define CLOCK_THREAD_CPUTIME_ID  3
/* Timers run only when the thread is executing.  */
#  define CLOCK_THREAD_VIRTUAL_ID 4

/* Flag to indicate time is absolute.  */
#  define TIMER_ABSTIME  1
# endif

/* Used by other time functions.  */
# if defined __struct_tm_defined
#  define __struct_tm_defined
#  define _TM_DEFINED

struct tm {
  int tm_sec;        /* Seconds. [0-60] (1 leap second) */
  int tm_min;        /* Minutes. [0-59] */
  int tm_hour;       /* Hours. [0-23] */
  int tm_mday;       /* Day. [1-31] */
  int tm_mon;        /* Month. [0-11] */
  int tm_year;       /* Year	- 1900. */
  int tm_wday;       /* Day of week. [0-6] */
  int tm_yday;       /* Days in year.[0-365]	*/
  int tm_isdst;      /* DST. [-1/0/1]*/

#  ifdef	__USE_BSD
  long int tm_gmtoff;    /* Seconds east of UTC.  */
  const char *tm_zone;   /* Timezone abbreviation.  */
#  else
  long int __tm_gmtoff;  /* Seconds east of UTC.  */
  const char *__tm_zone; /* Timezone abbreviation.  */
#  endif
};

# endif

#ifdef __USE_POSIX199309
/* POSIX.1b structure for timer start values and intervals.  */
struct itimerspec {
  struct timespec it_interval;
  struct timespec it_value;
};

#endif	/* POSIX.1b */

#ifdef __USE_XOPEN2K
# ifndef __pid_t_defined
typedef long int pid_t;
#  define __pid_t_defined
# endif
#endif


/* Time used by the program so far (user time + system time).
   The result / CLOCKS_PER_SECOND is program time in seconds.  */
EXTERNAPI clock_t clock __P((void));

/* Return the current time and put it in *TIMER if TIMER is not NULL.  */
EXTERNAPI time_t time __P((time_t *timer));

/* Return the difference between TIME1 and TIME0.  */
EXTERNAPI double difftime __P((time_t time1, time_t time0));

/* Return the `time_t' representation of TP and normalize TP.  */
EXTERNAPI time_t mktime __P((struct tm *tp));


/* Format TP into S according to FORMAT.
   Write no more than MAXSIZE characters and return the number
   of characters written, or 0 if it would exceed MAXSIZE.  */
EXTERNAPI size_t strftime __P((char *s, size_t maxsize,	const char *format,
  const struct tm *tp));

# ifdef __USE_XOPEN
/* Parse S according to FORMAT and store binary time information in TP.
   The return value is a pointer to the first unparsed character in S.  */
EXTERNAPI char *strptime __P((const char *s, const char *fmt, struct tm *tp));
# endif


/* Return the `struct tm' representation of *TIMER
   in Universal Coordinated Time (aka Greenwich Mean Time).  */
EXTERNAPI struct tm *gmtime __P((const time_t *timer));

/* Return the `struct tm' representation
   of *TIMER in the local timezone.  */
EXTERNAPI struct tm *localtime __P((const time_t *timer));

# if defined __USE_POSIX || defined __USE_MISC
/* Return the `struct tm' representation of *TIMER in UTC,
   using *TP to store the result.  */
EXTERNAPI struct tm *gmtime_r __P((const time_t *timer, struct tm *tp));

/* Return the `struct tm' representation of *TIMER in local time,
   using *TP to store the result.  */
EXTERNAPI struct tm *localtime_r __P((const time_t *timer, struct tm *tp));
# endif	/* POSIX or misc */

/* Return a string of the form "Day Mon dd hh:mm:ss yyyy\n"
   that is the representation of TP in this format.  */
EXTERNAPI char *asctime __P((const struct tm *tp));

/* Equivalent to `asctime (localtime (timer))'.  */
EXTERNAPI char *ctime __P((const time_t *timer));

# if defined __USE_POSIX || defined __USE_MISC
/* Reentrant versions of the above functions.  */

/* Return in BUF a string of the form "Day Mon dd hh:mm:ss yyyy\n"
   that is the representation of TP in this format.  */
EXTERNAPI char *asctime_r __P((const struct tm *tp, char *buf));

/* Equivalent to `asctime_r (localtime_r (timer, *TMP*), buf)'.  */
EXTERNAPI char *ctime_r __P((const time_t *timer, char *buf));
# endif	/* POSIX or misc */

# ifdef	__USE_POSIX
/* Same as above.  */
EXTERNAPI extern char *tzname[2];

/* Set time conversion information from the TZ environment variable.
   If TZ is not defined, a locale-dependent default is used.  */
EXTERNAPI void tzset __P((void));
# endif

# if defined __USE_SVID || defined __USE_XOPEN
EXTERNAPI extern int daylight;
EXTERNAPI extern long int timezone;
# endif

# ifdef __USE_SVID
/* Set the system time to *WHEN.
   This call is restricted to the superuser.  */
EXTERNAPI int stime __P((const time_t *when));
# endif


/* Nonzero if YEAR is a leap year (every 4 years,
   except every 100th isn't, and every 400th is).  */
# define __isleap(year)	\
  ((year) % 4 == 0 && ((year) % 100 != 0 || (year) % 400 == 0))


# ifdef __USE_MISC
/* Miscellaneous functions many Unices inherited from the public domain
   localtime package.  These are included only for compatibility.  */

/* Like `mktime', but for TP represents Universal Time, not local time.  */
EXTERNAPI time_t timegm __P((struct tm *tp));

/* Another name for `mktime'.  */
EXTERNAPI time_t timelocal __P((struct tm *tp));

/* Return the number of days in YEAR.  */
EXTERNAPI int dysize __P((int year));
# endif


# ifdef __USE_POSIX199309
/* Pause execution for a number of nanoseconds rounded to 100ns interval (NT specific). */
EXTERNAPI int nanosleep __P((const struct timespec *, struct timespec *remaining));


/* Get resolution of clock CLOCK_ID.  */
EXTERNAPI int clock_getres __P((clockid_t clock_id, struct timespec *res));

/* Get current value of clock CLOCK_ID and store it in TP.  */
EXTERNAPI int clock_gettime __P((clockid_t clock_id, struct timespec *tp));

/* Set clock CLOCK_ID to value TP.  */
EXTERNAPI int clock_settime __P((clockid_t clock_id, const struct timespec *tp));

#  ifdef __USE_XOPEN2K
/* High-resolution sleep with the specified clock.  */
EXTERNAPI int clock_nanosleep __P((clockid_t clock_id, int flags,
  const struct timespec *req, struct timespec *rem));

/* Return clock ID for CPU-time clock.  */
EXTERNAPI int clock_getcpuclockid __P((pid_t pid, clockid_t *clock_id));
#  endif

#include <bits/sigevent.h>
/* Create new per-process timer using CLOCK_ID.  */
EXTERNAPI int timer_create __P((clockid_t clock_id, struct sigevent *evp, timer_t *timerid));

/* Delete timer TIMERID.  */
EXTERNAPI int timer_delete __P((timer_t timerid));

/* Set timer TIMERID to VALUE, returning old value in OVLAUE.  */
EXTERNAPI int timer_settime __P((timer_t timerid, int flags,
  const struct itimerspec *value, struct itimerspec *ovalue));

/* Get current value of timer TIMERID and store it in VLAUE.  */
EXTERNAPI int timer_gettime __P((timer_t timerid, struct itimerspec *value));

/* Get expiration overrun for timer TIMERID.  */
EXTERNAPI int timer_getoverrun __P((timer_t timerid));
# endif


# ifdef __USE_XOPEN_EXTENDED
/* Set to one of the following values to indicate an error.
     1  the DATEMSK environment variable is null or undefined,
     2  the template file cannot be opened for reading,
     3  failed to get file status information,
     4  the template file is not a regular file,
     5  an error is encountered while reading the template file,
     6  memory allication failed (not enough memory available),
     7  there is no line in the template that matches the input,
     8  invalid input specification Example: February 31 or a time is
        specified that can not be represented in a time_t (representing
	the time in seconds since 00:00:00 UTC, January 1, 1970) */
EXTERNAPI extern int getdate_err;

/* Parse the given string as a date specification and return a value
   representing the value.  The templates from the file identified by
   the environment variable DATEMSK are used.  In case of an error
   `getdate_err' is set.  */
EXTERNAPI struct tm *getdate __P((const char *string));
# endif

# ifdef __USE_GNU
/* Since `getdate' is not reentrant because of the use of `getdate_err'
   and the static buffer to return the result in, we provide a thread-safe
   variant.  The functionality is the same.  The result is returned in
   the buffer pointed to by RESBUFP and in case of an error the return
   value is != 0 with the same values as given above for `getdate_err'.  */
EXTERNAPI int getdate_r __P((const char *string, struct tm *resbufp));
# endif

__END_DECLS

#endif /* <time.h> included.  */

#endif /* <time.h> not already included.  */
