/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 *	ISO C99 Standard: 7.25
 *	Wide character classification and mapping utilities  <wctype.h>
 */

#ifndef _WCTYPE_

#include <features.h>
#include <bits/types.h>

#ifndef __need_iswxxx
# define _WCTYPE_

/* We try to get wint_t from <stddef.h>, but not all GCC versions define it
   there.  So define it ourselves if it remains undefined.  */
# define __need_wint_t
# include <stddef.h>

# if !defined _WINT_T && !defined _WCTYPE_T_DEFINED
/* Integral type unchanged by default argument promotions that can
   hold any value corresponding to members of the extended character
   set, as well as at least one value that does not correspond to any
   member of the extended character set.  */
#  define _WINT_T
#  define _WCTYPE_T_DEFINED
typedef wchar_t wint_t;
/* Scalar type that can hold values which represent locale-specific
   character classifications.  */
typedef wchar_t wctype_t;
# endif

# define __WUPPER     0x0001      /* UPPERCASE */
# define __WLOWER     0x0002      /* lowercase */
# define __WDIGIT     0x0004      /* decimal digits */
# define __WSPACE     0x0008      /* spacing characters */
# define __WPUNCT     0x0010      /* punctuation characters */
# define __WCNTRL     0x0020      /* control characters */
# define __WBLANK     0x0040      /* blank characters */
# define __WXDIGIT    0x0080      /* other digits */
# define __WALPHA     0x0100      /* any linguistic character */

/* Constant expression of type `wint_t' whose value does not correspond
   to any member of the extended character set.  */
# ifndef WEOF
#  define WEOF (0xFFFFU)
# endif
#endif
#undef __need_iswxxx


/* The following part is also used in the <wcsmbs.h> header when compiled
   in the Unix98 compatibility mode.  */
#ifndef __iswxxx_defined
# define __iswxxx_defined

enum {
  _ISwupper = __WUPPER,                                   /* UPPERCASE. */
  _ISwlower = __WLOWER,                                   /* lowercase. */
  _ISwalpha = __WALPHA,                                   /* Alphabetic. */
  _ISwdigit = __WDIGIT,                                   /* Numeric. */
  _ISwxdigit = __WXDIGIT,                                 /* Hexadecimal numeric. */
  _ISwspace = __WSPACE,                                   /* Whitespace.  */
  _ISwprint = __WBLANK | __WPUNCT | __WALPHA | __WDIGIT,  /* Printing. */
  _ISwgraph = __WPUNCT | __WALPHA | __WDIGIT,             /* Graphical. */
  _ISwblank = __WBLANK,                                   /* Blank (usually SPC and TAB). */
  _ISwcntrl = __WCNTRL,                                   /* Control character. */
  _ISwpunct = __WPUNCT,                                   /* Punctuation. */
  _ISwalnum = __WALPHA | __WDIGIT,                        /* Alphanumeric. */
};

__BEGIN_DECLS

/*
 * Wide-character classification functions: 7.15.2.1.
 */

/* Test for any wide character for which `iswalpha' or `iswdigit' is
   true.  */
EXTERNAPI int iswalnum __P((wint_t __wc));

/* Test for any wide character for which `iswupper' or 'iswlower' is
   true, or any wide character that is one of a locale-specific set of
   wide-characters for which none of `iswcntrl', `iswdigit',
   `iswpunct', or `iswspace' is true.  */
EXTERNAPI int iswalpha __P((wint_t __wc));

/* Test for any control wide character.  */
EXTERNAPI int iswcntrl __P((wint_t __wc));

/* Test for any wide character that corresponds to a decimal-digit
   character.  */
EXTERNAPI int iswdigit __P((wint_t __wc));

/* Test for any wide character for which `iswprint' is true and
   `iswspace' is false.  */
EXTERNAPI int iswgraph __P((wint_t __wc));

/* Test for any wide character that corresponds to a lowercase letter
   or is one of a locale-specific set of wide characters for which
   none of `iswcntrl', `iswdigit', `iswpunct', or `iswspace' is true.  */
EXTERNAPI int iswlower __P((wint_t __wc));

/* Test for any printing wide character.  */
EXTERNAPI int iswprint __P((wint_t __wc));

/* Test for any printing wide character that is one of a
   locale-specific et of wide characters for which neither `iswspace'
   nor `iswalnum' is true.  */
EXTERNAPI int iswpunct __P((wint_t __wc));

/* Test for any wide character that corresponds to a locale-specific
   set of wide characters for which none of `iswalnum', `iswgraph', or
   `iswpunct' is true.  */
EXTERNAPI int iswspace __P((wint_t __wc));

/* Test for any wide character that corresponds to an uppercase letter
   or is one of a locale-specific set of wide character for which none
   of `iswcntrl', `iswdigit', `iswpunct', or `iswspace' is true.  */
EXTERNAPI int iswupper __P((wint_t __wc));

/* Test for any wide character that corresponds to a hexadecimal-digit
   character equivalent to that performed be the functions described
   in the previous subclause.  */
EXTERNAPI int iswxdigit __P((wint_t __wc));

/* Test for any wide character that corresponds to a standard blank
   wide character or a locale-specific set of wide characters for
   which `iswalnum' is false.  */
# ifdef __USE_ISOC99
EXTERNAPI int iswblank __P((wint_t __wc));
# endif
# ifdef __USE_MSVC_COMPATIBLE
EXTERNAPI int iswascii __P((wint_t __wc)); /* compatibility with MSVC */
# endif

/*
 * Extensible wide-character classification functions: 7.15.2.2.
 */

/* Construct value that describes a class of wide characters identified
   by the string argument PROPERTY.  */
EXTERNAPI wctype_t wctype __P((const char *__property));

/* Determine whether the wide-character WC has the property described by
   DESC.  */
EXTERNAPI int iswctype __P((wint_t __wc, wctype_t __desc));

# if defined NDEBUG && !defined _LIBC
/* The tables are always organized in a way which allows direct access
   for single byte characters.  */
EXTERNAPI extern wchar_t __wchar_ctypes[0xFFFF];

#  define iswalnum(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwalnum)
#  define iswalpha(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwalpha)
#  define iswcntrl(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwcntrl)
#  define iswdigit(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwdigit)
#  define iswlower(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwlower)
#  define iswgraph(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwgraph)
#  define iswprint(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwprint)
#  define iswpunct(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwpunct)
#  define iswspace(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwspace)
#  define iswupper(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwupper)
#  define iswxdigit(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwxdigit)
#  ifdef __USE_ISOC99
#   define iswblank(__wc) (__wchar_ctypes[(wchar_t)(__wc)] & _ISwblank)
#  endif
#  ifdef __USE_MSVC_COMPATIBLE
#   define iswascii(__wc) ((wchar_t)(__wc) < 0x80)
#  endif
#  define iswctype(__wc, __desc) (__wchar_ctypes[(wchar_t)(__wc)] & (wctype_t)(desc))
# endif
/*
 * Wide-character case-mapping functions: 7.15.3.1.
 */

/* Scalar type that can hold values which represent locale-specific
   character mappings.  */
typedef const wint_t *wctrans_t;

/* Converts an uppercase letter to the corresponding lowercase letter.  */
EXTERNAPI wint_t towlower __P((wint_t __wc));

/* Converts an lowercase letter to the corresponding uppercase letter.  */
EXTERNAPI wint_t towupper __P((wint_t __wc));

__END_DECLS

#endif	/* need iswxxx.  */

/* The remaining definitions and declarations must not appear in the
   <wcsmbs.h> header.  */
#ifdef _WCTYPE_

/*
 * Extensible wide-character mapping functions: 7.15.3.2.
 */

__BEGIN_DECLS

/* Construct value that describes a mapping between wide characters
   identified by the string argument PROPERTY.  */
EXTERNAPI wctrans_t wctrans __P((const char *__property));

/* Map the wide character WC using the mapping described by DESC.  */
EXTERNAPI wint_t towctrans __P((wint_t __wc, wctrans_t __desc));

# ifdef __USE_GNU
/* Declare the interface to extended locale model.  */
#  include <xlocale.h>

/* Test for any wide character for which `iswalpha' or `iswdigit' is
   true.  */
EXTERNAPI int __iswalnum_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character for which `iswupper' or 'iswlower' is
   true, or any wide character that is one of a locale-specific set of
   wide-characters for which none of `iswcntrl', `iswdigit',
   `iswpunct', or `iswspace' is true.  */
EXTERNAPI int __iswalpha_l __P((wint_t __wc, __locale_t __locale));

/* Test for any control wide character.  */
EXTERNAPI int __iswcntrl_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to a decimal-digit
   character.  */
EXTERNAPI int __iswdigit_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character for which `iswprint' is true and
   `iswspace' is false.  */
EXTERNAPI int __iswgraph_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to a lowercase letter
   or is one of a locale-specific set of wide characters for which
   none of `iswcntrl', `iswdigit', `iswpunct', or `iswspace' is true.  */
EXTERNAPI int __iswlower_l __P((wint_t __wc, __locale_t __locale));

/* Test for any printing wide character.  */
EXTERNAPI int __iswprint_l __P((wint_t __wc, __locale_t __locale));

/* Test for any printing wide character that is one of a
   locale-specific et of wide characters for which neither `iswspace'
   nor `iswalnum' is true.  */
EXTERNAPI int __iswpunct_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to a locale-specific
   set of wide characters for which none of `iswalnum', `iswgraph', or
   `iswpunct' is true.  */
EXTERNAPI int __iswspace_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to an uppercase letter
   or is one of a locale-specific set of wide character for which none
   of `iswcntrl', `iswdigit', `iswpunct', or `iswspace' is true.  */
EXTERNAPI int __iswupper_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to a hexadecimal-digit
   character equivalent to that performed be the functions described
   in the previous subclause.  */
EXTERNAPI int __iswxdigit_l __P((wint_t __wc, __locale_t __locale));

/* Test for any wide character that corresponds to a standard blank
   wide character or a locale-specific set of wide characters for
   which `iswalnum' is false.  */
EXTERNAPI int __iswblank_l __P((wint_t __wc, __locale_t __locale));

/* Construct value that describes a class of wide characters identified
   by the string argument PROPERTY.  */
EXTERNAPI wctype_t __wctype_l __P((const char *__property, __locale_t __locale));

/* Determine whether the wide-character WC has the property described by
   DESC.  */
EXTERNAPI int __iswctype_l __P((wint_t __wc, wctype_t __desc, __locale_t __locale));

/*
 * Wide-character case-mapping functions.
 */

/* Converts an uppercase letter to the corresponding lowercase letter.  */
EXTERNAPI wint_t __towlower_l __P((wint_t __wc, __locale_t __locale));

/* Converts an lowercase letter to the corresponding uppercase letter.  */
EXTERNAPI wint_t __towupper_l __P((wint_t __wc, __locale_t __locale));

/* Construct value that describes a mapping between wide characters
   identified by the string argument PROPERTY.  */
EXTERNAPI wctrans_t __wctrans_l __P((const char *__property, __locale_t __locale));

/* Map the wide character WC using the mapping described by DESC.  */
EXTERNAPI wint_t __towctrans_l __P((wint_t __wc, wctrans_t __desc, __locale_t __locale));

#define __iswalnum_l(__wc, __locale) iswalnum (__wc)
#define __iswalpha_l(__wc, __locale) iswalpha (__wc)
#define __iswcntrl_l(__wc, __locale) iswcntrl (__wc)
#define __iswdigit_l(__wc, __locale) iswdigit (__wc)
#define __iswlower_l(__wc, __locale) iswlower (__wc)
#define __iswgraph_l(__wc, __locale) iswgraph (__wc)
#define __iswprint_l(__wc, __locale) iswprint (__wc)
#define __iswpunct_l(__wc, __locale) iswpunct (__wc)
#define __iswspace_l(__wc, __locale) iswspace (__wc)
#define __iswupper_l(__wc, __locale) iswupper (__wc)
#define __iswxdigit_l(__wc, __locale) iswxdigit (__wc)
#define __iswblank_l(__wc, __locale) iswblank (__wc)
#define __wctype_l(__property, __locale) wctype (__property)
#define __iswctype_l(__wc, __desc, __locale) iswctype (__wc, __desc)
#define __towlower_l(__wc, __locale) towlower (__wc)
#define __towupper_l(__wc, __locale) towupper (__wc)
#define __wctrans_l(__property, __locale) wctrans (__property)
#define __towctrans_l(__wc, __desc, __locale) towctrans (__wc, __desc)

# endif /* Use GNU.  */

__END_DECLS

#endif	/* _WCTYPE_ defined.  */

#endif /* wctype.h  */