/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <internal.h>
#include <syscalls.h>

STATIC NTSTATUS __SysEnter ();
STATIC NTSTATUS __Int2E ();
NTSTATUS (*SysCall) () = NULL;

NAKED NTSTATUS __SysEnter ()
{
  __asm {
    mov edx, esp
    _emit 0x0F
    _emit 0x34
    ret
  }
}

NAKED NTSTATUS __Int2E ()
{
  __asm {
    lea edx, dword ptr [esp + 8]
    int 0x2E
    ret
  }
}

BOOLEAN ReadDriverInformation (VOID)
{
  STATIC WCHAR DriverName[] = L"\\Device\\psxsys";

  HANDLE File;
  IO_STATUS_BLOCK IoStatusBlock;
  OBJECT_ATTRIBUTES ObjectAttrib;
  NTSTATUS Status;
  UNICODE_STRING UnicodeName;
  BOOLEAN FastSysCall;

  UnicodeName.MaximumLength = sizeof DriverName;
  UnicodeName.Length = UnicodeName.MaximumLength - 2;
  UnicodeName.Buffer = DriverName;

  ObjectAttrib.Length = sizeof ObjectAttrib;
  ObjectAttrib.Attributes = OBJ_CASE_INSENSITIVE;
  ObjectAttrib.ObjectName = &UnicodeName;
  ObjectAttrib.RootDirectory = NULL;
  ObjectAttrib.SecurityDescriptor = NULL;
  ObjectAttrib.SecurityQualityOfService = NULL;

  Status = NtCreateFile (&File, SYNCHRONIZE | FILE_READ_DATA,
    &ObjectAttrib, &IoStatusBlock, 0, FILE_ATTRIBUTE_NORMAL, FILE_SHARE_READ,
    FILE_OPEN, FILE_SYNCHRONOUS_IO_NONALERT, NULL, 0);

  if (Status >= 0) {
    Status = NtDeviceIoControlFile (File, NULL, NULL, NULL, &IoStatusBlock,
      PSX_IS_FAST_SYSCALL, NULL, 0, &FastSysCall, sizeof FastSysCall);

    if (Status >= 0) {
      if (FastSysCall)
        SysCall = __SysEnter;
      else
        SysCall = __Int2E;
    }

    NtClose (File);
  }

  return (Status >= 0);
}

#ifdef _WIN64
# pragma pack (push, 8)
#else
# pragma pack (push, 4)
#endif

NAKED NTSTATUS NtPsxCreateThread (
  PULONG ThreadId,
  PHANDLE HandleAddr OPTIONAL,
  PCTHREAD_ATTRIBUTES Attrib,
  VOID (*StartRoutine)(PVOID, PVOID),
  PVOID UserArgument1,
  PVOID UserArgument2
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSetCallback (
  int CancelIndex,
  int Signal1Index,
  int Signal2Index,
  int TimerThreadIndex
  )
{
  __asm {
    mov eax, _SYSCALL_SET_CALLBACK
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxExitThread (
  PVOID Exitvalue
  )
{
  __asm {
    mov eax, _SYSCALL_EXIT_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxJoinThread (
  ULONG ThreadId,
  PVOID *ExitValue
  )
{
  __asm {
    mov eax, _SYSCALL_JOIN_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDetachThread (
  ULONG ThreadId
  )
{
  __asm {
    mov eax, _SYSCALL_DETACH_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadSetScheduler (
  ULONG ThreadId,
  int Policy,
  const struct sched_param *SchedParam
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_SET_SCHEDULER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadGetScheduler (
  ULONG ThreadId,
  int *Policy,
  struct sched_param *SchedParam
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_GET_SCHEDULER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadSetCancelState (
  int State,
  int *OldState
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_SET_CANCEL_STATE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadGetCancelState (
  int *OldState
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_GET_CANCEL_STATE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadSetCancelType (
  int Type,
  int *TypeAddr
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_SET_CANCEL_TYPE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadGetCancelType (
  int *TypeAddr
  )
{
  __asm {
    mov eax, _SYSCALL_THREAD_GET_CANCEL_TYPE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCancelThread (
  ULONG ThreadId
  )
{
  __asm {
    mov eax, _SYSCALL_CANCEL_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadTestCancel (VOID)
{
  __asm {
    mov eax, _SYSCALL_THREAD_TEST_CANCEL
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxWaitThread (VOID)
{
  __asm {
    mov eax, _SYSCALL_WAIT_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxTimedWaitThread (
  const struct timespec *TimeSpec
  )
{
  __asm {
    mov eax, _SYSCALL_TIMEDWAIT_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxUnWaitThread (
  ULONG ThreadId
  )
{
  __asm {
    mov eax, _SYSCALL_UNWAIT_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxKillThread (
  ULONG ThreadId,
  int Signal,
  siginfo_t *UInfo OPTIONAL
  )
{
  __asm {
    mov eax, _SYSCALL_KILL_THREAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCreateCondVar (
  OUT PHANDLE CondVarId,
  IN BOOLEAN Shared
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDestroyCondVar (
  HANDLE CondVarId
  )
{
  __asm {
    mov eax, _SYSCALL_DESTROY_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxWaitCondVar (
  HANDLE CondVarId,
  PMUTANT Mutant
  )
{
  __asm {
    mov eax, _SYSCALL_WAIT_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxTimedWaitCondVar (
  HANDLE CondVarId,
  PMUTANT Mutant,
  const struct timespec *TimeSpec
  )
{
  __asm {
    mov eax, _SYSCALL_TIMEDWAIT_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSignalCondVar (
  HANDLE CondVarId
  )
{
  __asm {
    mov eax, _SYSCALL_SIGNAL_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxBroadCastCondVar (
  HANDLE CondVarId
  )
{
  __asm {
    mov eax, _SYSCALL_BROADCAST_CONDVAR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCreateWaitObj (
  OUT PHANDLE WaitObjId,
  IN BOOLEAN Shared
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_WAITOBJ
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDestroyWaitObj (
  HANDLE WaitObjId
  )
{
  __asm {
    mov eax, _SYSCALL_DESTROY_WAITOBJ
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxWaitObject (
  HANDLE WaitObjId
  )
{
  __asm {
    mov eax, _SYSCALL_WAIT_WAITOBJ
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxTimedWaitObject (
  HANDLE WaitObjId,
  const struct timespec *TimeSpec
  )
{
  __asm {
    mov eax, _SYSCALL_TIMEDWAIT_WAITOBJ
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSignalWaitObj (
  HANDLE WaitObjId,
  int Count
  )
{
  __asm {
    mov eax, _SYSCALL_SIGNAL_WAITOBJ
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxKill (
  LONG PiD,
  int Signal,
  const union sigval Value,
  BOOLEAN Queue,
  siginfo_t *UInfo
  )
{
  __asm {
    mov eax, _SYSCALL_KILL
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSignalMask (
  int How,
  const sigset_t *SignalMask,
  sigset_t *OldSignalMask
  )
{
  __asm {
    mov eax, _SYSCALL_SIGNAL_MASK
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxHoldSignal (
  int Signal
  )
{
  __asm {
    mov eax, _SYSCALL_HOLD_SIGNAL
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSignalAction (
  int Signal,
  const struct sigaction *Action,
  struct sigaction *OldAction
  )
{
  __asm {
    mov eax, _SYSCALL_SIGNAL_ACTION
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxPause (
  const struct timespec *TimeSpec
  )
{
  __asm {
    mov eax, _SYSCALL_PAUSE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxPending (
  sigset_t *PendingMask
  )
{
  __asm {
    mov eax, _SYSCALL_PENDING
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxWaitSignal (
  const sigset_t *Mask,
  siginfo_t *Value,
  const struct timespec *TimeSpec OPTIONAL
  )
{
  __asm {
    mov eax, _SYSCALL_WAITSIGNAL
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSignalVector (
  int Signal,
  const struct sigvec *Vector,
  struct sigvec *OldVector
  )
{
  __asm {
    mov eax, _SYSCALL_SIGNAL_VECTOR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSetPosixSignalMode (VOID)
{
  __asm {
    mov eax, _SYSCALL_POSIX_SIGNAL_MODE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxClock (
  clock_t *ClockTicks
  )
{
  __asm {
    mov eax, _SYSCALL_CLOCK
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGetClockResolution (
  clockid_t	ClockId,
  struct timespec *Resolution
  )
{
  __asm {
    mov eax, _SYSCALL_GET_TIMER_RESOLUTION
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGetKUTime (
  PULONG Kernel,
  PULONG User
  )
{
  __asm {
    mov eax, _SYSCALL_GET_KUTIME
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxIntervalTimer (
  int Which,
  struct itimerval *Value,
  struct itimerval *OldValue
  )
{
  __asm {
    mov eax, _SYSCALL_INTERVAL_TIMER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGet86Vector (
  int Index,
  PGATE32 Gate,
  PUSHORT IdtSize
  )
{
  __asm {
    mov eax, _SYSCALL_GET86_VECTOR
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGet86PageEntry (
  SIZE_T Address,
  PPAGE_ENTRY Pde,
  PPAGE_ENTRY Pte
  )
{
  __asm {
    mov eax, _SYSCALL_GET86_PAGEENTRY
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGet86VAD (
  SIZE_T Virtual,
  PVAD UVad
  )
{
  __asm {
    mov eax, _SYSCALL_GET86_VAD
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtIx86Disassemble (
  CONST UCHAR **CommandBuffer,
  SIZE_T BufferSize,
  PIX86_COMMAND Cmd,
  PSIZE_T CmdCount,
  UCHAR BitnessMode,
  CONST CHAR *StopAtCommand
  )
{
  __asm {
    mov eax, _SYSCALL_IX86_DISASSEMBLE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxSetScheduler (
  ULONG TaskId,
  int Policy,
  const struct sched_param *SchedParam
  )
{
  __asm {
    mov eax, _SYSCALL_SET_SCHEDULER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGetScheduler (
  ULONG TaskId,
  int *Policy,
  struct sched_param *SchedParam
  )
{
  __asm {
    mov eax, _SYSCALL_GET_SCHEDULER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCreateBarrier (
  OUT PHANDLE BarrierId,
  IN LONG Count,
  IN BOOLEAN Shared
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_BARRIER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDestroyBarrier (
  HANDLE BarrierId
  )
{
  __asm {
    mov eax, _SYSCALL_DESTROY_BARRIER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxWaitBarrier (
  HANDLE BarrierId
  )
{
  __asm {
    mov eax, _SYSCALL_WAIT_BARRIER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCreateWaitObjEx (
  OUT PHANDLE WaitObjId,
  IN LONG SemValue,
  IN BOOLEAN Shared
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_WAITOBJ_EX
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDestroyWaitObjEx (
  HANDLE WaitObjId,
  ULONG Flags
  )
{
  __asm {
    mov eax, _SYSCALL_DESTROY_WAITOBJ_EX
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGetWaitObjValue (
  HANDLE WaitObjId,
  PLONG Value
  )
{
  __asm {
    mov eax, _SYSCALL_GET_WAITOBJ_VALUE
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxThreadGetKUTime (
  PULONG Kernel,
  PULONG User
  )
{
  __asm {
    mov eax, _SYSCALL_GET_THREAD_KUTIME
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxCreateTimer (
  clockid_t ClockId,
  struct sigevent *Event,
  VOID (*StartRoutine)(PVOID, PVOID),
  timer_t *TimerId
  )
{
  __asm {
    mov eax, _SYSCALL_CREATE_TIMER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxTimer (
  timer_t TimerId,
  int Flags,
  struct itimerspec *Value,
  struct itimerspec *OValue
  )
{
  __asm {
    mov eax, _SYSCALL_TIMER
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxGetTimerOverrun (
  timer_t TimerId,
  int *Overrun
  )
{
  __asm {
    mov eax, _SYSCALL_GET_TIMER_OVERRUN
    call dword ptr SysCall
    ret
  }
}

NAKED NTSTATUS NtPsxDeleteTimer (
  timer_t TimerId
  )
{
  __asm {
    mov eax, _SYSCALL_DELETE_TIMER
    call dword ptr SysCall
    ret
  }
}

/////////////////////////////////////////////

__declspec (naked) clock_t clock (void)
{
  __asm {
    sub esp, TYPE clock_t
    push esp
    push 0
    mov eax, _SYSCALL_CLOCK
    call dword ptr SysCall
    add esp, 8
    pop eax
    pop edx
    ret
  }
}

#pragma pack (pop)
