/*
 * WINNT POSIX Library
 * Version:  1.0
 * 
 * Copyright (C) 2002  Andrew Gnezdilov avi@risp.ru  All Rights Reserved.
 *
 * WINNT POSIX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * WINNT POSIX is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WINNT POSIX; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _SYSCALLS_
#define _SYSCALLS_

#include <psxdisasm.h>

#ifdef _WIN64
# pragma pack (push, 8)
#else
# pragma pack (push, 4)
#endif

#define _SYSTABLE_OFFSET  0x3000

typedef enum _SYSCALL_INDEX {
  _SYSCALL_CREATE_THREAD = _SYSTABLE_OFFSET,
  _SYSCALL_SET_CALLBACK,
  _SYSCALL_EXIT_THREAD,
  _SYSCALL_JOIN_THREAD,
  _SYSCALL_DETACH_THREAD,
  _SYSCALL_THREAD_SET_SCHEDULER,
  _SYSCALL_THREAD_GET_SCHEDULER,
  _SYSCALL_THREAD_SET_CANCEL_STATE,
  _SYSCALL_THREAD_SET_CANCEL_TYPE,
  _SYSCALL_CANCEL_THREAD,
  _SYSCALL_THREAD_TEST_CANCEL,
  _SYSCALL_WAIT_THREAD,
  _SYSCALL_TIMEDWAIT_THREAD,
  _SYSCALL_UNWAIT_THREAD,
  _SYSCALL_KILL_THREAD,
  _SYSCALL_CREATE_CONDVAR,
  _SYSCALL_DESTROY_CONDVAR,
  _SYSCALL_WAIT_CONDVAR,
  _SYSCALL_TIMEDWAIT_CONDVAR,
  _SYSCALL_SIGNAL_CONDVAR,
  _SYSCALL_BROADCAST_CONDVAR,
  _SYSCALL_CREATE_WAITOBJ,
  _SYSCALL_DESTROY_WAITOBJ,
  _SYSCALL_WAIT_WAITOBJ,
  _SYSCALL_TIMEDWAIT_WAITOBJ,
  _SYSCALL_SIGNAL_WAITOBJ,
  _SYSCALL_KILL,
  _SYSCALL_SIGNAL_MASK,
  _SYSCALL_HOLD_SIGNAL,
  _SYSCALL_SIGNAL_ACTION,
  _SYSCALL_PAUSE,
  _SYSCALL_PENDING,
  _SYSCALL_WAITSIGNAL,
  _SYSCALL_SIGNAL_VECTOR,
  _SYSCALL_POSIX_SIGNAL_MODE,
  _SYSCALL_CLOCK,
  _SYSCALL_GET_TIMER_RESOLUTION,
  _SYSCALL_GET_KUTIME,
  _SYSCALL_INTERVAL_TIMER,
  _SYSCALL_GET86_VECTOR,
  _SYSCALL_GET86_PAGEENTRY,
  _SYSCALL_GET86_VAD,
  _SYSCALL_IX86_DISASSEMBLE,
  _SYSCALL_SET_SCHEDULER,
  _SYSCALL_GET_SCHEDULER,
  _SYSCALL_THREAD_GET_CANCEL_STATE,
  _SYSCALL_THREAD_GET_CANCEL_TYPE,
  _SYSCALL_CREATE_BARRIER,
  _SYSCALL_DESTROY_BARRIER,
  _SYSCALL_WAIT_BARRIER,
  _SYSCALL_CREATE_WAITOBJ_EX,
  _SYSCALL_DESTROY_WAITOBJ_EX,
  _SYSCALL_GET_WAITOBJ_VALUE,
  _SYSCALL_GET_THREAD_KUTIME,
  _SYSCALL_CREATE_TIMER,
  _SYSCALL_TIMER,
  _SYSCALL_GET_TIMER_OVERRUN,
  _SYSCALL_DELETE_TIMER,
  _SYSCALL_YIELD_EXECUTION,
  _SYSCALL_MAX
} SYSCALL_INDEX, *PSYSCALL_INDEX;


NTSTATUS NtPsxCreateThread (
  PULONG ThreadId,
  PHANDLE HandleAddr OPTIONAL,
  PCTHREAD_ATTRIBUTES Attrib,
  VOID (*StartRoutine)(PVOID, PVOID),
  PVOID UserArgument1,
  PVOID UserArgument2
  );

typedef struct _SIGNAL_CALLBACK_BUFFER_1 {
  void (*signal_handler) ();
  int signal_number;
  int signal_code;
  sigcontext context;
} SIGNAL_CALLBACK_BUFFER_1, *PSIGNAL_CALLBACK_BUFFER_1;

typedef struct _SIGNAL_CALLBACK_BUFFER_2 {
  void (*signal_handler) ();
  siginfo_t signal_info;
  ucontext_t ucontext;
} SIGNAL_CALLBACK_BUFFER_2, *PSIGNAL_CALLBACK_BUFFER_2;

typedef struct _TIMER_THREAD_CALLBACK_BUFFER {
  HANDLE thread_handle;
  HANDLE thread_id;
  mcontext_t *ctx;
} TIMER_THREAD_CALLBACK_BUFFER, *PTIMER_THREAD_CALLBACK_BUFFER;

#ifdef _PSXSYS_

typedef struct _SIGNAL_FRAME_2 {
  SIGNAL_CALLBACK_BUFFER_2 buffer;
  mcontext_t ctx;
} SIGNAL_FRAME_2, *PSIGNAL_FRAME_2;

typedef struct _SIGNAL_FRAME_1 {
  SIGNAL_CALLBACK_BUFFER_1 buffer;
  mcontext_t ctx;
} SIGNAL_FRAME_1, *PSIGNAL_FRAME_1;

typedef struct _TIMER_THREAD_FRAME {
  TIMER_THREAD_CALLBACK_BUFFER buffer;
  mcontext_t ctx;
} TIMER_THREAD_FRAME, *PTIMER_THREAD_FRAME;

#endif

NTSTATUS NtPsxSetCallback (
  int CancelIndex,
  int Signal1Index,
  int Signal2Index,
  int TimerThreadIndex
  );

NTSTATUS NtPsxExitThread (
  PVOID ExitValue
  );

NTSTATUS NtPsxJoinThread (
  ULONG ThreadId,
  PVOID *ExitValue
  );

NTSTATUS NtPsxDetachThread (
  ULONG ThreadId
  );

NTSTATUS NtPsxThreadSetScheduler (
  ULONG ThreadId,
  int Policy,
  const struct sched_param *SchedParam
  );

NTSTATUS NtPsxThreadGetScheduler (
  ULONG ThreadId,
  int *Policy,
  struct sched_param *SchedParam
  );

NTSTATUS NtPsxThreadSetCancelState (
  int State,
  int *OldState
  );

NTSTATUS NtPsxThreadGetCancelState (
  int *StateAddr
  );

NTSTATUS NtPsxThreadSetCancelType (
  int Type,
  int *OldType
  );

NTSTATUS NtPsxThreadGetCancelType (
  int *OldType
  );

NTSTATUS NtPsxCancelThread (
  ULONG ThreadId
  );

NTSTATUS NtPsxThreadTestCancel (VOID);

NTSTATUS NtPsxWaitThread (VOID);

NTSTATUS NtPsxTimedWaitThread (
  const struct timespec *Timespec
  );

NTSTATUS NtPsxUnWaitThread (
  ULONG ThreadId
  );

NTSTATUS NtPsxKillThread (
  ULONG ThreadId,
  int Signal,
  siginfo_t *UInfo OPTIONAL
  );

NTSTATUS NtPsxCreateCondVar (
  OUT PHANDLE CondVarId,
  IN BOOLEAN Shared
  );

NTSTATUS NtPsxDestroyCondVar (
  HANDLE CondVarId
  );

NTSTATUS NtPsxWaitCondVar (
  HANDLE CondVarId,
  PMUTANT Mutant
  );

NTSTATUS NtPsxTimedWaitCondVar (
  HANDLE CondVarId,
  PMUTANT Mutant,
  const struct timespec *TimeSpec
  );

NTSTATUS NtPsxSignalCondVar (
  HANDLE CondVarId
  );

NTSTATUS NtPsxBroadCastCondVar (
  HANDLE CondVarId
  );

NTSTATUS NtPsxCreateWaitObj (
  OUT PHANDLE WaitObjId,
  IN BOOLEAN Shared
  );

NTSTATUS NtPsxDestroyWaitObj (
  HANDLE WaitObjId
  );

NTSTATUS NtPsxWaitObject (
  HANDLE WaitObjId
  );

NTSTATUS NtPsxTimedWaitObject (
  HANDLE WaitObjId,
  const struct timespec *TimeSpec
  );

NTSTATUS NtPsxSignalWaitObj (
  HANDLE WaitObjId,
  int Count
  );

NTSTATUS NtPsxKill (
  LONG PiD,
  int Signal,
  const union sigval Value,
  BOOLEAN Queue,
  siginfo_t *UInfo
  );

NTSTATUS NtPsxSignalMask (
  int How,
  const sigset_t *SignalMask,
  sigset_t *OldSignalMask
  );

NTSTATUS NtPsxHoldSignal (
  int Signal
  );

NTSTATUS NtPsxSignalAction (
  int Signal,
  const struct sigaction *Action,
  struct sigaction *OldAction
  );

NTSTATUS NtPsxPause (
  const struct timespec *TimeSpec
  );

NTSTATUS NtPsxPending (
  sigset_t *PendingMask
  );

NTSTATUS NtPsxWaitSignal (
  const sigset_t *Mask,
  siginfo_t *Value,
  const struct timespec *TimeSpec OPTIONAL
  );

NTSTATUS NtPsxSignalVector (
  int Signal,
  const struct sigvec *Vector,
  struct sigvec *OldVector
  );

NTSTATUS NtPsxSetPosixSignalMode (VOID);

NTSTATUS NtPsxClock (
  clock_t *ClockTicks
  );

NTSTATUS NtPsxGetClockResolution (
  clockid_t	ClockId,
  struct timespec *Resolution
  );

NTSTATUS NtPsxGetKUTime (
  PULONG Kernel,
  PULONG User
  );

NTSTATUS NtPsxIntervalTimer (
  int Which,
  struct itimerval *Value,
  struct itimerval *OldValue
  );

#pragma pack (push, 1)

typedef union _GATE32 {
  struct _GATE32_BITS {
    USHORT LowWord;
    USHORT Segment;
    UCHAR Parameters : 5;
    UCHAR Reserved : 3;
    UCHAR Type : 4;
    UCHAR S : 1;
    UCHAR Dpl : 2;
    UCHAR P : 1;
    USHORT HighWord;
  };
  ULONGLONG Single;
} GATE32, *PGATE32;

typedef union _PDE4M {
  struct _PDE4M_BITS {
    ULONG P : 1;
    ULONG RW : 1;
    ULONG US : 1;
    ULONG PWT : 1;
    ULONG PCD : 1;
    ULONG A : 1;
    ULONG D : 1;
    ULONG PS : 1;
    ULONG G : 1;
    ULONG Available : 3;
    ULONG Reserved : 10;
    ULONG PFN : 10;
  };
  ULONG Single;
} PDE4M, *PPDE4M;

typedef union _PDE4K {
  struct _PDE4K_BITS {
    ULONG P : 1;
    ULONG RW : 1;
    ULONG US : 1;
    ULONG PWT : 1;
    ULONG PCD : 1;
    ULONG A : 1;
    ULONG Reserved : 1;
    ULONG PS : 1;
    ULONG G : 1;
    ULONG Available : 3;
    ULONG PFN : 20;
  };
  ULONG Single;
} PDE4K, *PPDE4K;

typedef union _PTE4K {
  struct _PTE4K_BITS {
    ULONG P : 1;
    ULONG RW : 1;
    ULONG US : 1;
    ULONG PWT : 1;
    ULONG PCD : 1;
    ULONG A : 1;
    ULONG D : 1;
    ULONG Reserved : 1;
    ULONG G : 1;
    ULONG Available : 3;
    ULONG PFN : 20;
  };
  ULONG Single;
} PTE4K, *PPTE4K;

// GUARD | NOCACHE ==> NO ACCESS

typedef union _PAGEDPTE {
  struct _PAGED_BITS {
    ULONG P : 1;
    ULONG PageFileNumber : 4;
    ULONG WriteCopy : 1;
    ULONG Execute : 1;
    ULONG Write : 1;
    ULONG NoCache : 1;
    ULONG Guard : 1;
    ULONG ProtoPte : 1;
    ULONG TransitionPte : 1;
    ULONG FileOffset : 20;
  };
  ULONG Single;
} PAGEDPTE, *PPAGEDPTE;

typedef union _TRANSITIONPTE {
  struct _TRANSITION_BITS {
    ULONG P : 1;
    ULONG Write2 : 1;
    ULONG Owner : 1;
    ULONG WriteThrouth : 1;
    ULONG DisableCache : 1;
    ULONG WriteCopy : 1;
    ULONG Execute : 1;
    ULONG Write : 1;
    ULONG NoCache : 1;
    ULONG Guard : 1;
    ULONG ProtoPte : 1;
    ULONG TransitionPte : 1;
    ULONG FileOffset : 20;
  };
  ULONG Single;
} TRANSITIONPTE, *PTRANSITIONPTE;

#pragma pack (pop)

typedef union _PAGE_ENTRY {
  PDE4K Pde4K;
  PDE4M Pde4M;
  PTE4K Pte4K;
  PAGEDPTE Paged;
  TRANSITIONPTE Transition;
  ULONG Single;
} PAGE_ENTRY, *PPAGE_ENTRY;

NTSTATUS NtPsxGet86Vector (
  int Index,
  PGATE32 Gate,
  PUSHORT IdtSize
  );

NTSTATUS NtPsxGet86PageEntry (
  SIZE_T Address,
  PPAGE_ENTRY Pde,
  PPAGE_ENTRY Pte
  );

#define _VAD_GUARD             1
#define _VAD_RESERVE_COMMITED  4
#define _VAD_RESERVED          8
#define _VAD_COMMIT_READONLY   1
#define _VAD_EXECUTE           2
#define _VAD_COMMIT_WRITE      4
#define _VAD_NOCACHE           8

#define VAD_GUARD              0x10000000
#define VAD_RESERVE_COMMITED_1 0x40000000
#define VAD_COMMIT_1_MASK      0x40FFFFFE
#define VAD_RESERVED           0x80000000
#define VAD_COMMIT_READ_1      0x1000000
#define VAD_COMMIT_WRITE_2     0x1000000
#define VAD_EXECUTE            0x2000000
#define VAD_COMMIT_WRITE_1     0x4000000
#define VAD_COMMIT_READ_2      0x4000000
#define VAD_NOCACHE            0x8000000
#define VAD_RESERVE_COMMITED_2 0x00000001
#define VAD_SECTION_GUARD      0x00000001
#define VAD_NOACESS            (VAD_GUARD | VAD_NOCACHE)

/* if VAD_RESERVE_COMMITED_2 set then xVAD_x_x_2
 * else if any of VAD_COMMIT_1_MASK set then aplied xVAD_x_x_1
 */

typedef struct _VAD VAD, *PVAD;
struct _VAD {
  SIZE_T StartAddress;
  SIZE_T EndAddress;
  PVAD ParentLink;
  PVAD LeftLink;
  PVAD RightLink;
  union _ATTRIB {
    struct _BITS {
      ULONG Unknown : 24;
      ULONG CommitAttrib : 4;
      ULONG ReserveAttrib : 4;
    };
    ULONG Attrib;
  };
};

NTSTATUS NtPsxGet86VAD (
  SIZE_T Address,
  PVAD Vad
  );

NTSTATUS NtIx86Disassemble (
  CONST UCHAR **CommandBuffer,
  SIZE_T BufferSize,
  PIX86_COMMAND Cmd,
  PSIZE_T CmdCount,
  UCHAR BitnessMode,
  CONST CHAR *StopAtCommand
  );

NTSTATUS NtPsxSetScheduler (
  ULONG TaskId,
  int Policy,
  const struct sched_param *SchedParam
  );

NTSTATUS NtPsxGetScheduler (
  ULONG TaskId,
  int *Policy,
  struct sched_param *SchedParam
  );

NTSTATUS NtPsxCreateBarrier (
  OUT PHANDLE BarrierId,
  IN LONG Count,
  IN BOOLEAN Shared
  );

NTSTATUS NtPsxDestroyBarrier (
  HANDLE BarrierId
  );

NTSTATUS NtPsxWaitBarrier (
  HANDLE BarrierId
  );

NTSTATUS NtPsxCreateWaitObjEx (
  OUT PHANDLE WaitObjId,
  IN LONG SemValue,
  IN BOOLEAN Shared
  );

#define _DESTROY_BUSY_LOCK 1

NTSTATUS NtPsxDestroyWaitObjEx (
  HANDLE WaitObjId,
  ULONG Flags
  );

NTSTATUS NtPsxGetWaitObjValue (
  HANDLE WaitObjId,
  PLONG Value
  );

NTSTATUS NtPsxThreadGetKUTime (
  PULONG Kernel,
  PULONG User
  );

NTSTATUS NtPsxCreateTimer (
  clockid_t ClockId,
  struct sigevent *Event,
  VOID (*StartRoutine)(PVOID, PVOID),
  timer_t *TimerId
  );

NTSTATUS NtPsxTimer (
  timer_t TimerId,
  int Flags,
  struct itimerspec *Value,
  struct itimerspec *OValue
  );

NTSTATUS NtPsxGetTimerOverrun (
  timer_t TimerId,
  int *Overrun
  );

NTSTATUS NtPsxDeleteTimer (
  timer_t TimerId
  );

#ifdef _PSXSYS_KERNEL_
NTSTATUS NtPsxYieldExecution (VOID);
#endif

#define PSX_DEVICE 0x8000
#define PSX_IS_FAST_SYSCALL 0X80006000

#ifndef _PSXSYS_KERNEL_
BOOLEAN ReadDriverInformation (VOID);
extern NTSTATUS (*SysCall) ();
#endif

#pragma pack (pop)

#endif